
/*****************************************************************************/
/*   Copyright 1998 National Instruments Corporation.  All Rights Reserved.  */
/*****************************************************************************/

#include <utility.h> 
#include <visa.h>
#include <ansi_c.h>
#include <formatio.h>
#include "avr3267s.h"

#define avr3267s_REVISION	"Rev 1.1, 10/2000, CVI 5.0" /* Instrument driver revision */
#define BUFFER_SIZE			1024L         				/* File I/O buffer size */
    
/*=============== R346x Series - Modulation Spectrum Analyzes ===============*/
/* LabWindows/CVI 4.0.1 Instrument Driver                                    */
/* Original Release: January 1998                                            */
/* By: Martin Koutny         Email: martin.koutny@vsb.cz		             */
/*     PH. +420 69 699 4882  Fax: +420 69 691 9597                           */
/*                                                                           */
/* Modification History: 		 											 */
/*      April 2000: Modifyed for R3267s by Advantest SPS project  (V 1.0 )	 */
/*===========================================================================*/

/*=============== R3267/73 Series Spectrum Analyzer =========================*/
/* LabWindows/CVI 5.0 Instrument Driver										 */
/* Original Release: April/2000	( V 1.0 )									 */
/* By: Hitoshi Takahashi	Supervisor: Yohei Hirakoso						 */
/*                   		SPSpj Gyoda R&D,  ADVANTEST CORPORATION			 */
/*                   		Home Page; http://www.advantest.co.jp			 */
/*                   		e-mail;    info@advantest.co.jp					 */
/*                                                                           */
/* Modification History:													 */
/*     Version 1.1 : Oct/5/00 Modifyed by Advantest SPS project				 */
/*                   - Two Functions Addition								 */
/*                     ( Operation Mode, Communication System )				 */
/*																			 */
/*===========================================================================*/


/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/

ViString oprModeArr[] = {"SETFUNC CW", "SETFUNC TRAN"};
ViString commSysArr[] = {"COMMSYS WCDMA", "COMMSYS IS95", "COMMSYS PDC",
					     "COMMSYS PHS", "COMMSYS IS136", "COMMSYS GSM",
						 "COMMSYS DECT", "COMMSYS FMDEV", "COMMSYS 3GPP",
						 "COMMSYS BLUETOOTH", "COMMSYS CDMA2000", "COMMSYS 64QAM"};
ViString unitsArr[] = {"GZ","MZ","KZ","HZ","MV","MW","DB","MA","SC","MS","US"};
ViString frqSpanArr[] = {"SP","FS","ZS","LS"};
ViString preselArr[] = {"PRESL STD","PRESL EXTD", ""};
ViString linearArr[] = {"LL1","LL2","LL5","LL10"};
ViString levelUnitsArr[] = {"AUNITS DBM","AUNITS DBMV","AUNITS DBUV","AUNITS DBEMF","AUNITS DBPW","AUNITS W","AUNITS V"};
ViString dBrateArr[] = {"DD 10","DD 5","DD 2","DD 1","DD 0.5"}; 
ViString sweepModeArr[] = {"SI","SN"};
ViString sweepWinArr[] = {"WDOSWP OFF","WDOSWP ON"};
ViString gatedSweepArr[] = {"GTSWP OFF","GTSWP ON"};
ViString gateSrcArr[] = {"GTSRC IF","GTSRC EGT","GTSRC EXT"};
ViString sweepActionArr[] = {"SR","TS","SN","SI"};
ViString trgModeArr[] = {"TRGSRC FREE","TRGSRC LINE","TRGSRC VIDEO","TRGSRC EXT","TRGSRC IF"};
ViString trigSlopeArr[] = {"TRGSLP FALL","TRGSLP RISE"};
ViString detTrcAModeArr[] = {"DET NRM","DET POS","DET NEG","DET SMP"};
ViString detTrcBModeArr[] = {"DETB NRM","DETB POS","DETB NEG","DETB SMP"}; 
ViString tracePointArr[] = {"TPS","TPL"};
ViString traceAArr[] = {"AW","AV","AB","AM","AMIN ON","AGR","APAVG ON","CWA"};
ViString aveModeArr[] = {"","AGR","AGS","AGP","AGC","AG1","AG0"};
ViString traceBArr[] = {"BW","BV","BB","BM","BMIN ON","BGR","BPABG ON","CWB"};
ViString traceMathArr[] = {"ACHB","TR0","TR1","TR2"};
ViString PFJudgeArr[] = {"PFC OFF","PFC ON"};
ViString limLine1Arr[] = {"LMTA OFF","LMTA ON"};
ViString limLine2Arr[] = {"LMTB OFF","LMTB ON"};
ViString ln1PassRngArr[] = {"LARNG BELOW","LARNG ABOVE"};
ViString ln2PassRngArr[] = {"LBRNG BELOW","LBRNG ABOVE"};
ViString xAxisArr[] = {"LIMPOS REL","LIMPOS ABS"};   
ViString yAxisArr[] = {"LIMAPOS REL","LIMAPOS ABS"};
ViString xRefArr[] = {"LIMXREF LEFT","LIMXREF CENT","LIMXREF UDEF"};
ViString yRefArr[] = {"LIMYREF TOP","LIMYREF BOTM","LIMYREF UDEF"};
ViString limitJudgArr[] = {"PFJ?","OPF?","FPU?","FPL?"};
ViString rdMkrXArr[] = {"MF?","MDF1?","MDF2?"};
ViString rdMkrLvlArr[] = {"ML?","MDL1?","MDL2?"};
ViString dispLineArr[] = {"DL OFF","DL ON"};
ViString mkrModeArr[] = {"MKOFF","MKN","MKD"};
ViString othMkrArr[] = {"FX ON","FX OFF","REDLT ON","REDLT OFF",
						"MKTRACE TRA", "MKTRACE TRB"};
ViString sigTrackArr[] = {"SGF","SGN"};
ViString refObjArr[] = {"CPLMK OFF","CPLMK DLT","CPLMK ANC","CPLMK LLIN","CPLMK DLIN"};
ViString peakActArr[] = {"PS","NXP","NXL","NXR","MIS","NXM"};
ViString contPeakArr[] = {"CP OFF","CP ON"};
ViString peakXRaArr[] = {"MKSX ALL","MKSX IN","MKSX OUT"};
ViString peakYRaArr[] = {"MKSY ALL","MKSY DLIN","MKSY LLIN",
						"MKSY DLIN;MKSYDL ABOVE","MKSY DLIN;MKSYDL BELOW",
						"MKSY LLIN;MKSYLA ABOVE","MKSY LLIN;MKSYLA BELOW",
						"MKSY LLIN;MKSYLB ABOVE","MKSY LLIN;MKSYLB BELOW"};
ViString cpltoFTArr[] = {"MKSCPL OFF","MKSCPL ON"};
ViString multiMkrArr[] = {"MLT OFF","MLT ON","MLN1","MLN2","MLN3","MLN4","MLN5",
						 "MLN6","MLN7","MLN8","MLN9","MLN10","MLF1",
						 "MLF2","MLF3","MLF4","MLF5","MLF6","MLF7",
						 "MLF8","MLF9","MLF10"};
ViString peakListArr[] = {"PLS FREQ","PLS LEVEL","MO"};
ViString markerToArr[] = {"MC","MR","DS","M0","M1","MTCF","M2","M3","PKCF","PKRL"};
ViString dBDownArr[] = {"MKBW","XDB","XDL","XDR","PS;XDB","DC0","DC1","DC2","CDB ON","CDB OFF"};
ViString modAMArr[] = {"AMMOD OFF","AMMOD ON"}; 
ViString windStateArr[] = {"WDO OFF","WDO ON"};
ViString windSweepArr[] = {"WDOSWP OFF","WDOSWP ON"};
ViString multiScrArr[] = {"MLTSCR ZM","MLTSCR FT","MLTSCR TT","MLTSCR OFF"};
ViString pkZmArr[] = {"","PKZMX","PKZMN","PKZM3"};
ViString scrSideArr[] = {"SCRSEL TRB","SCRSEL TRA"};
ViString levelCorrArr[] = {"CR OFF","CR ON","","CRDEL"};
ViString recallArr[] = {"RC","SV","DEL"};
ViString copyModeArr[] = {"PRT GRY","PRT MOS","PRT MOL"};
ViString prnCommArr[] = {"PRTCMD ESC","PRTCMD PCL","PRTCMD ESCR"};
ViString pltObjArr[] = {"PLTRACE","PLALL"};
ViString splitArr[] = {"PLPIC1","PLPIC2","PLPIC4"};
ViString plotPosArr[] = {"PLMID","PLLEFT","PLRIGHT","PLUPLEFT","PLUPRIGHT",
						"PLLOWLEFT","PLLOWRIGHT"};
ViString locModeArr[] = {"PLMAN","PLAUTO"};
ViString addrModeArr[] = {"PLTALK ADRS","PLTALK ONLY"};
ViString imgModeArr[] = {"HCIMAG COL","HCIMAG GRY","HCIMAG MON"};
ViString RLEArr[] = {"HCCMPRS OFF","HCCMPRS ON"};
ViString setAutoArr[] = {"BA","VA","AS","CA","MPA"};
ViString setManualArr[] = {"RB","VB","SW","CS","MPM"};
ViString setMnUnitArr[] = {"HZ","HZ","SC","HZ",""};
ViString bandLockArr[] = {"BNDLC OFF","BNDLC ON"};
ViString lossvsFreqArr[] = {"LVF OFF","LVF ON"};
ViString deviceSelArr[] = {"HCDEV PRT","HCDEV MA","HCDEV MB","HCDEV FDD"};
ViString freqCorrArr[] = {"FC OFF","FC ON"};
ViString calCorrArr[] = {"CC OFF","CC ON"};
ViString calFuncArr[] = {"CLALL","CLGAIN","CLATT","CLSTEP","CLRBW","CLLOG",
						"CLMAG","CLPBW","CLN","CLCREF","CLFREF"};
ViString memCardInitArr[] = {"MMI /B: /","MMI /A: /"};
ViString driveSelArr[] = {"DEV /RAM: /","DEV /A: /","DEV /B: /"};
ViString labelArr[] = {"LOF","LON"};
ViString memoryArr[] = {"TBB","TBA"};
ViString powMeasArr[] = {"PWAVG","PWTOTAL","PWCH","PWM"};
ViString freqDataArr[] = {"CF","SP","FA","FB"};
ViString cntResArr[] = {"CN0","CN1","CN2","CN3"};
ViString cntModeArr[] = {"COUNT OFF","COUNT ON"};
ViString squelchArr[] = {"SQE OFF","SQE ON"};
ViString soundModeArr[] = {"SOF","SFM","SAM","SON"};
ViString noiseModeArr[] = {"NIM","NIU","NIC","NIF"};
ViString setUpArr[] = {"USR","DEF","MNL","STD"};
ViString screenArr[] = {"ACPSCR SEPA","ACPSCR FULL"};
ViString graphArr[] = {"ADG OFF","ADG ON"};
ViString sprMeasArr[] = {"SPURI OFF","SPURI ON"};
ViString XYAnchorArr[] = {"","SETANC","RLSANC"};
ViString ServiceReqArr[] = {"S1","S0"};
ViString uniqueWordArr[] = {"UNIQ NO","UNIQ B16","UNIQ B32"};
ViString commRateArr[] = {"CODEC FULL","CODEC HALF"};
ViString autoLevelArr[] = {"ALS OFF","ALS ON"};
ViString commSystemArr[] = {"MODTYP PDC","MODTYP PHS","MODTYP NADC"};
ViString functionArr[] = {"TAVGOBW","TAVGACP","TAVGMOD","TAVGTR","TAVGAP","TAVGCR"};
ViString triggerModeArr[] = {"TRGMODE AUTO","TRGMODE EXT","TRGMODE SOFT"};
ViString trgWFSrcArr[] = {"TRGSRC FREE","TRGSRC VIDEO","TRGSRC IF","TRGSRC EXT"};
ViString trgSlopeArr[] = {"FALL","RISE"};
ViString srcMonArr[] = {"TRGMON OFF","TRGMON ON"};
ViString modeArr[] = {"AUTO","EXT"};
ViString TZoomArr[] = {"DCRST","DCZOM"};
ViString spuriousModeArr[] = {"RCLSTBL","SAVSTBL","SPUFREQ","DELSTBL"};
ViString measFuncArr[] = {"WAVEFM","SPCT","TOBW","TACP","SPUR","TXPWR",
						 "OFFPWR","MODACC","BTR","IBSPR","SI"};
ViString executionArr[] = {"AUTOWFL","AUTOLVL"};
ViString measValueArr[] = {"SPULVL?","IBSPR?"};
ViString CDMAcommSystemArr[] = {"MODTYP CDMA800M","MODTYP CDMA1700M","MODTYP CDMA1900M"};
ViString commLinkArr[] = {"LINK FWD","LINK REV"};
ViString signalRateArr[] = {"RATE 9600","RATE 4800","RATE 2400","RATE 1200"};
ViString triggerSourceArr[] = {"TGTSRC IF","TGTSRC EXT"};
ViString saveParametersArr[] = {"","TGTSV"};
ViString switchArr[] = {"OFF","ON"};
ViString YScaleArr[] = {"CPWDIV P5DB","CPWDIV P10DB"};
ViString refPowerArr[] = {"COBWREF DSP","COBWREF SWP"};
ViString traceDetectorArr[] = {"COBWDET SMP","COBWDET POS"};
ViString measModeArr[] = {"SPRMOD REL","SPRMOD ABS"};
ViString refPowArr[] = {"SPRREF DSP","SPRREF SWP"};
ViString traceDetArr[] = {"SPRDET SMP","SPRDET POS"};
ViString withScreenArr[] = {"SPRSCRN AB","SPRSCRN A","SPRSCRN B"};
ViString WQmeasModeArr[] = {"WMMOD NZERO","WMMOD PILOT"};
ViString tauOffsetArr[] = {"USER","DFLT"};
ViString WQtrigSourceArr[] = {"WTRGSRC EXT","WTRGSRC INT"};
ViString limitTypeArr[] = {"WLMTYP USER","WLMTYP STD"};
ViString YScaleTypeArr[] = {"CYSCL RHO","CYSCL POW"};
ViString YScaledBdivArr[] = {"CPDIV P10DB","CPDIV P5DB"};
ViString CDPmeasModeArr[] = {"CMMOD PREC","CMMOD NORM"};
ViString CPDtrigSourceArr[] = {"CTRGSRC EXT","CTRGSRC INT"};
ViString TXtrigSourceArr[] = {"TTRGSRC EXT","TTRGSRC INT"};
ViString avgFunctionArr[] = {"TAVGBST","CAVGGAT","CAVGRAT","CAVGOBW","CAVGCHP",
							"CAVGSPR","CAVGWF","CAVGCDP","CTXAVG"};
ViString zoomWindowArr[] = {"DCHZOM ON","DCHZOM OFF","DCHZOM BURST","DCHZOM FRAME",
						   "DCZOM", "DCRST"};
ViString DYScaleArr[] = {"DCVDIV P10DB","DCVDIV P5DB","DCVDIV P2DB"};
ViString graphTypeArr[] = {"WFGTYP CON","WFGTYP CONLIN","WFGTYP CONDOT",
						  "WFGTYP ICHEYE","WFGTYP QCHEYE","WFGTYP MAGERR",
						  "WFGTYP PHAERR","WFGTYP EVM"};
ViString funcMeasureArr[] = {"PWRTIME","GATEPOW","RATIO","COBW","CCHPOW",
							"CINBSPR","WFQUA","WFGPH","CDPGPH","CDPRES;SI",
							"CTXPOW","SI"};
ViString typeSelectionArr[] = {"TM1","TM2","FR1","FR2","FR3","FR4","FR5"};
ViString tableTypeArr[] = {"LMCPSL STD","LMCPSL USR"};
ViString basicTypeArr[] = {"WFRHO","WFRHO","WFRHO","WFMAG","WFPHSE","WFMOD","CODPOW"};
ViString xDataTypeArr[] = {"GPHI","GPHCHIP","GPHCHIP","GPHCHIP","GPHCHIP",
						  "GPHCHIP","GPHCH"};
ViString yDataTypeArr[] = {"GPHQ","GPHI","GPHQ","GPHERR","GPHDEG","GPHERR","GPHRHO"};
ViString GMRArr[] = {"GMKQ","GMKI","GMKQ","GMKERR","GMKDEG","GMKERR"};
ViString gateSweepArr[] = {"TGTSWP OFF","TGTSWP ON"};

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* avr3267s_stringValPair is used in the avr3267s_errorMessage function        */
/* avr3267s_statusDataRanges is used to track session dependent status & ranges*/
/*===========================================================================*/
typedef struct  avr3267s_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  avr3267s_tStringValPair;
 
struct avr3267s_statusDataRanges {
    ViInt16 traceRange;
    ViInt16 harmonics;
    ViChar instrDriverRevision[256];
};

typedef struct avr3267s_statusDataRanges *avr3267s_instrRange;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean avr3267s_invalidViBooleanRange (ViBoolean val);
ViBoolean avr3267s_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean avr3267s_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean avr3267s_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max);
ViBoolean avr3267s_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean avr3267s_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean avr3267s_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean avr3267s_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus avr3267s_waitOnVisaEvent (ViSession instrSession, ViEventType eventTypeIn,
	ViUInt32 tmoIn, ViPUInt16 STB);
ViStatus avr3267s_initCleanUp (ViSession openRMSession, ViPSession openInstrSession,
	ViStatus currentStatus);
ViStatus avr3267s_defaultInstrSetup (ViSession openInstrSession);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_init (ViRsrc resourceName, ViBoolean IDQuery,
                    ViBoolean resetDevice, ViPSession instrSession)
{
    ViStatus avr3267s_status = VI_SUCCESS;
    ViSession rmSession = 0;
    ViUInt32 retCnt = 0;
    ViByte rdBuffer[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/
    if (avr3267s_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViBooleanRange (resetDevice))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/
    if ((avr3267s_status = viOpenDefaultRM (&rmSession)) < 0)
        return avr3267s_status;

    if ((avr3267s_status = viOpen (rmSession, resourceName, VI_NULL, VI_NULL, instrSession)) < 0) {
        viClose (rmSession);
        return avr3267s_status;
    }

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    if ((avr3267s_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 10000)) < 0)
            return avr3267s_initCleanUp (rmSession, instrSession, avr3267s_status);
  
    if ((avr3267s_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return avr3267s_initCleanUp (rmSession, instrSession, avr3267s_status);
  
    if ((avr3267s_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return avr3267s_initCleanUp (rmSession, instrSession, avr3267s_status);
    
    if ((avr3267s_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return avr3267s_initCleanUp (rmSession, instrSession, avr3267s_status);

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery) {
        if ((avr3267s_status = viWrite (*instrSession, "*IDN?", 5, &retCnt)) < 0)
            return avr3267s_initCleanUp (rmSession, instrSession, avr3267s_status);
        if ((avr3267s_status = viRead (*instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
            return avr3267s_status;

        Scan (rdBuffer, "ADVANTEST,R3267");
        if (NumFmtdBytes () != 15) 
        {
        	Scan (rdBuffer, "ADVANTEST,R3273");
        	if (NumFmtdBytes () != 15)       
            	return avr3267s_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
        }
    }
        
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice) {
        if ((avr3267s_status = avr3267s_reset (*instrSession)) < 0)
            return avr3267s_initCleanUp (rmSession, instrSession, avr3267s_status);
    }       
    else  /*- Send Default Instrument Setup ---------------------------------*/
        if ((avr3267s_status = avr3267s_defaultInstrSetup (*instrSession)) < 0)
            return avr3267s_initCleanUp (rmSession, instrSession, avr3267s_status);
          
    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Measurement of Frequency                                        */
/* Purpose:  This function ilustrates sample measurement of approximately 30 */
/*           MHz signal source with normal marker and reading data trace	 */
/*           memory for advanced processing.			 					 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_appFrqMeas (ViSession instrSession,
                ViReal64 centerFrequency, ViReal64 frequencySpan,
                ViPReal64 markerFrequency, ViPReal64 markerLevel,
                ViInt16 _VI_FAR traceDataArray[])
{
    ViStatus avr3267s_status = VI_SUCCESS;
    ViInt16 units;

    if (avr3267s_invalidViReal64Range (centerFrequency, 0.0, 26.5e9))
	    return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (frequencySpan, 0.0, 26.5e9))
        return VI_ERROR_PARAMETER3;

    /* Set up measurement parameters */
    if ((avr3267s_status = avr3267s_confCenterFrq (instrSession, centerFrequency,
    	VI_ON, 0.0, VI_OFF, 0.0)) < 0)
        return avr3267s_status;
    if ((avr3267s_status = avr3267s_confFrqSpan (instrSession, 0, frequencySpan)) < 0)
        return avr3267s_status;
    if ((avr3267s_status = avr3267s_actPeakSearch (instrSession, 0, VI_OFF)) < 0)
        return avr3267s_status;
    /* Read out measurement results */
    if ((avr3267s_status = avr3267s_actMkrData (instrSession, markerFrequency, markerLevel,
    	&units)) < 0)
        return avr3267s_status;
    if ((avr3267s_status = avr3267s_dataTraceOut (instrSession, VI_ON, traceDataArray)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Operation Mode		                                             */
/* Purpose:  This function selects the operation mode.						 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_setOperationMode (ViSession instrSession, ViBoolean operationMode)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViBooleanRange (operationMode))
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuf, "%s<%s", oprModeArr[operationMode]);    
    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Communication System                                            */
/* Purpose:  This function selects the communication system.				 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_setCommunicationSystem (ViSession instrSession, ViInt16 commSystem)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (commSystem, 0, 11))
        return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuf, "%s<%s", commSysArr[commSystem]);    
    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Center Frequency                                                */
/* Purpose:  This function sets the center frequency setting mode.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confCenterFrq (ViSession instrSession, ViReal64 centerFrequency,
					ViBoolean CFStep, ViReal64 CFStepSize, ViBoolean freqOffset,
            	    ViReal64 freqOffsetSize)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViReal64Range (centerFrequency, 0.0, 26.5e9))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViBooleanRange (CFStep))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViReal64Range (CFStepSize, 0.0, 26.5e9))
        return VI_ERROR_PARAMETER4;
    if (avr3267s_invalidViBooleanRange (freqOffset))
        return VI_ERROR_PARAMETER5;
    if (avr3267s_invalidViReal64Range (freqOffsetSize, -100.0e6, 100.0e6))
        return VI_ERROR_PARAMETER6;

	if (freqOffset == VI_OFF)                             
		Fmt (wrtBuf, "%s<FO OFF");                    
	else                                                  
		Fmt (wrtBuf, "%s<FO ON %fHZ", freqOffsetSize); 
	total += NumFmtdBytes();                              

    Fmt (wrtBuf, "%s[a]<;CF %fHZ", centerFrequency);
    total += NumFmtdBytes();
    
    if (CFStep == VI_OFF)
    	Fmt (wrtBuf, "%s[a]<;CS %fHZ", CFStepSize);
	else
		Fmt (wrtBuf, "%s[a]<;CA");
    total += NumFmtdBytes();
    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Frequency Span                                                  */
/* Purpose:  This function sets the frequency span setting mode.             */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confFrqSpan (ViSession instrSession, ViInt16 spanMode,
                	ViReal64 frequencySpan)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (spanMode, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (frequencySpan, 0.0, 26.5e9))
        return VI_ERROR_PARAMETER3;

    if (spanMode == 0)
    	Fmt (wrtBuf, "%s<%s %fHZ", frqSpanArr[spanMode], frequencySpan);
    else
    	Fmt (wrtBuf, "%s<%s", frqSpanArr[spanMode]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Start/Stop Frequency                                            */
/* Purpose:  This function sets the start/stop frequency setting mode.       */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confStartStopFrq (ViSession instrSession, ViReal64 startFrequency,
    	            ViReal64 stopFrequency)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViReal64Range (startFrequency, 0.0, 26.5e9))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (stopFrequency, 0.0, 26.5e9))
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "%s<FA %fHZ;FB %fHZ", startFrequency, stopFrequency);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}
    	            
/*===========================================================================*/
/* Function: Frequency Pre-selection                                         */
/* Purpose:  This function adjusts a tracking between the sweep frequency of */
/*           the preselector embedded in the input part and the sweep input  */
/*           frequency in the input frequency range of 1.7 to 8.2 GHz. This  */
/*           function does not work with R3463.								 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confFrqPreSel (ViSession instrSession, ViInt16 preselector,
        	        ViBoolean peaking, ViInt32 manPeakVal)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;
		 
    if (avr3267s_invalidViInt16Range (preselector, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViBooleanRange (peaking))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViInt32Range (manPeakVal, -100, 100))
        return VI_ERROR_PARAMETER4;

    Fmt (wrtBuf, "%s<%s", preselArr[preselector]);
    total += NumFmtdBytes();
    if (peaking == VI_OFF)
		Fmt (wrtBuf, "%s[a]<;PPM %i[b4]", manPeakVal);
	else
		Fmt (wrtBuf, "%s[a]<;PPA");
    total += NumFmtdBytes();

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Frequency Setup                                                 */
/* Purpose:  This function sets center frequency, span, start frequency and  */
/*           stop frequency.												 */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_confFreqSetup (ViSession instrSession, ViInt16 contents, 
									ViReal64 centerstart, ViReal64 spanstop)
{
	ViUInt32 retCnt = 0;                         
	ViChar wrtBuf[BUFFER_SIZE];                  
	ViStatus avr3267s_status = VI_SUCCESS;
	
	if (avr3267s_invalidViInt16Range (contents, 0, 5))          
	    return VI_ERROR_PARAMETER2;
	if (avr3267s_invalidViReal64Range (centerstart, 0.0, 26.5e9))       
	    return VI_ERROR_PARAMETER3;
	if (avr3267s_invalidViReal64Range (spanstop, 0.0, 26.5e9))
	    return VI_ERROR_PARAMETER3;
	
	switch (contents)
	{
	case 0:
		Fmt (wrtBuf, "%s<CF %fHZ", centerstart);
		break;
		
	case 1:
		Fmt (wrtBuf, "%s<SP %fHZ", spanstop);
		break;
		
	case 2:
		Fmt(wrtBuf, "%s<CF %fHZ;SP %fHZ", centerstart, spanstop);
		break;
		
	case 3:
		Fmt (wrtBuf, "%s<FA %fHZ", centerstart);              
		break;                                                
	
	case 4:
		Fmt (wrtBuf, "%s<FB %fHZ", spanstop);       
		break;                                      
	
	case 5:
		Fmt(wrtBuf, "%s<CF %fHZ;SP %fHZ", centerstart, spanstop);   
		break;                                                      
	}
	
	if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)      
	    return avr3267s_status;                                                      
	                                                                                 
	return avr3267s_status;                                                          
}

/*===========================================================================*/
/* Function: Reference Level                                                 */
/* Purpose:  This function sets the reference level parameters.				 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confRefLevel (ViSession instrSession,
                    ViReal64 referenceLevel, ViInt16 xdBdiv,
                    ViInt16 linear, ViInt16 units, ViBoolean levelOffset,
                    ViReal64 levelOffsetValue)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (units == 0) {
    	if (avr3267s_invalidViReal64Range (referenceLevel, -149.9, 60.0))
        	return VI_ERROR_PARAMETER2;
    }
    if (avr3267s_invalidViInt16Range (xdBdiv, 0, 4))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViInt16Range (linear, 0, 3))
        return VI_ERROR_PARAMETER4;
    if (avr3267s_invalidViInt16Range (units, 0, 6))
        return VI_ERROR_PARAMETER5;
    if (avr3267s_invalidViBooleanRange (levelOffset))
        return VI_ERROR_PARAMETER6;
    if (units == 0) {
	    if (avr3267s_invalidViReal64Range (levelOffsetValue, -100.0, 100.0))
    	    return VI_ERROR_PARAMETER7;
    }

    if (levelOffset == VI_ON)
    	Fmt (wrtBuf, "%s<%s;RL %f;RON %f",levelUnitsArr[units],
    		referenceLevel, levelOffsetValue);
    else
    	Fmt (wrtBuf, "%s<%s;ROF;RL %f", levelUnitsArr[units], referenceLevel);
    total += NumFmtdBytes();
    if (units == 6) {
    	Fmt (wrtBuf, "%s[a]<;%s", linearArr[linear]);
    }
    else {
        Fmt (wrtBuf, "%s[a]<;%s", dBrateArr[xdBdiv]);
    }
    total += NumFmtdBytes();

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Sweep Mode                                                      */
/* Purpose:  This function sets sweep mode functions such as single sweep,   */
/*           normal sweep, gated sweep and their parameters.				 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confSweepMode (ViSession instrSession, ViBoolean sweepMode,
                    ViBoolean window, ViReal64 gatePosition,
                    ViReal64 gateWidth, ViBoolean gatedSweep, ViInt16 gateSource)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViBooleanRange (sweepMode))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViBooleanRange (window))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViReal64Range (gatePosition, 0.0, 1.0))
        return VI_ERROR_PARAMETER4;
    if (avr3267s_invalidViReal64Range (gateWidth, 1.0e-6, 1.0))
        return VI_ERROR_PARAMETER5;
    if (avr3267s_invalidViBooleanRange (gatedSweep))
        return VI_ERROR_PARAMETER6;
    if (avr3267s_invalidViInt16Range (gateSource, 0, 2))
        return VI_ERROR_PARAMETER8;
 
 	Fmt (wrtBuf, "%s<GTSWP OFF;%s", sweepWinArr[window]);
 	total += NumFmtdBytes ();
 	
 	if (window==VI_OFF)
 	{
 		Fmt (wrtBuf, "%s[a]<;%s", gateSrcArr[gateSource]);
 		total += NumFmtdBytes ();
 		
 		if (gateSource!=1)
 		{
 			Fmt (wrtBuf, "%s[a]<;GTPOS %f;GTWID %f", gatePosition, gateWidth);
 			total += NumFmtdBytes ();
 		}
 		
 		Fmt (wrtBuf, "%s[a]<;%s", gatedSweepArr[gatedSweep]);
 		total += NumFmtdBytes ();
 	}
 	
 	Fmt (wrtBuf, "%s[a]<;%s", sweepModeArr[sweepMode]);
 	total += NumFmtdBytes ();
 	
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Trigger Mode                                                    */
/* Purpose:  This function sets the trigger conditions.						 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confTriggerMode (ViSession instrSession, ViInt16 triggerMode,
                    ViInt16 TrgLevel, ViBoolean triggerSlope, 
                    ViBoolean triggerDelay, ViReal64 delayTime)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;
    ViUInt32 total = 0;

    if (avr3267s_invalidViInt16Range (triggerMode, 0, 4))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (TrgLevel, 0.0, 100.0))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViBooleanRange (triggerSlope))
        return VI_ERROR_PARAMETER4;
    if (avr3267s_invalidViBooleanRange (triggerDelay))    
        return VI_ERROR_PARAMETER5;                       
    if (avr3267s_invalidViReal64Range (delayTime, -120.0e-3, 1.0e3))
    	return VI_ERROR_PARAMETER6;

    Fmt (wrtBuf, "%s<%s", trgModeArr[triggerMode]);
    total += NumFmtdBytes();
    
    if (triggerMode >= 2)
    {
    	Fmt (wrtBuf, "%s[a]<;%s", trigSlopeArr[triggerSlope]);
    	total += NumFmtdBytes();
    }
    	
    if (triggerMode==2 || triggerMode==4)
    {
    	Fmt (wrtBuf, "%s[a]<;TRGLVL %d", TrgLevel);
    	total += NumFmtdBytes();
    }
    
    if (triggerDelay==VI_ON)
    {
    	Fmt (wrtBuf, "%s[a]<;TRGDT %f", delayTime);
    	total += NumFmtdBytes();
    }

   if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Sweep Time                                                      */
/* Purpose:  This function sets the sweep time period.						 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confSweepTime (ViSession instrSession, ViBoolean sweepTimeMode,
                    ViReal64 sweepTimeValue)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViBooleanRange (sweepTimeMode))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (sweepTimeValue, 1.0e-6, 1000.0))
        return VI_ERROR_PARAMETER3;
    
	if (sweepTimeMode == VI_ON)
	    Fmt (wrtBuf, "%s<AS");
	else
		Fmt (wrtBuf, "%s<SW %f", sweepTimeValue);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Resolution Bandwidth                                            */
/* Purpose:  This function sets the Resolution Bandwidth (RBW).				 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confResBW (ViSession instrSession, ViBoolean RBWMode,
                    ViReal64 RBWFrqValue, ViBoolean RBWSpan,
                    ViReal64 RBWSpanValue)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViBooleanRange (RBWMode))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (RBWFrqValue, 10.0, 10.0e6))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViBooleanRange (RBWSpan))
        return VI_ERROR_PARAMETER4;
    if (avr3267s_invalidViReal64Range (RBWSpanValue, 0.001, 0.1))
        return VI_ERROR_PARAMETER5;

    if (RBWMode == VI_ON)
    	Fmt (wrtBuf, "%s<BA");
	else
		Fmt (wrtBuf, "%s<RB %fHZ", RBWFrqValue);
    total += NumFmtdBytes();
    
    if (RBWSpan == VI_ON)
    	Fmt (wrtBuf, "%s[a]<;CORS ON %f", RBWSpanValue);
	else
		Fmt (wrtBuf, "%s[a]<;CORS OFF");
    total += NumFmtdBytes();

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Video Bandwidth                                                 */
/* Purpose:  This function sets the Video Bandwidth (VBW).					 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confVideoBW (ViSession instrSession, ViBoolean VBWMode,
                    ViReal64 VBWFrqValue, ViBoolean VBWSpan,
                    ViReal64 VBWSpanValue)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViBooleanRange (VBWMode))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (VBWFrqValue, 1.0, 10.0e6))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViBooleanRange (VBWSpan))
        return VI_ERROR_PARAMETER4;
    if (avr3267s_invalidViReal64Range (VBWSpanValue, 0.003, 3.0))
        return VI_ERROR_PARAMETER5;

    if (VBWMode == VI_ON)
    	Fmt (wrtBuf, "%s<VA");
	else
		Fmt (wrtBuf, "%s<VB %fHZ", VBWFrqValue);
    total += NumFmtdBytes();
    
    if (VBWSpan == VI_ON)
    	Fmt (wrtBuf, "%s[a]<;COVR ON %f", VBWSpanValue);
	else
		Fmt (wrtBuf, "%s[a]<;COVR OFF");
    total += NumFmtdBytes();

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Input Attenuator                                                */
/* Purpose:  This function sets input attenuator parameters.				 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confATT (ViSession instrSession, ViBoolean ATTLevel,
                    ViReal64 ATTLevelValue, ViBoolean minimumATT,
                    ViReal64 minATTValue)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViBooleanRange (ATTLevel))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (ATTLevelValue, 0.0, 75.0))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViBooleanRange (minimumATT))
        return VI_ERROR_PARAMETER4;
    if (avr3267s_invalidViReal64Range (minATTValue, 0.0, 75.0))
        return VI_ERROR_PARAMETER5;

    if (ATTLevel == VI_ON)
    	Fmt (wrtBuf, "%s<AA");
	else
		Fmt (wrtBuf, "%s<AT %fDB", ATTLevelValue);
    total += NumFmtdBytes();

    if (minimumATT == VI_ON)
    	Fmt (wrtBuf, "%s[a]<;ATMIN ON %fDB", minATTValue);
	else
		Fmt (wrtBuf, "%s[a]<;ATMIN OFF");
    total += NumFmtdBytes();

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}


/*===========================================================================*/
/* Function: Trace Detector                                                  */
/* Purpose:  This function sets Trace detector mode.						 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confTraceDetector (ViSession instrSession, ViBoolean trace, ViInt16 detectorMode)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

	if (avr3267s_invalidViBooleanRange (trace))        
	    return VI_ERROR_PARAMETER2;                       
    if (avr3267s_invalidViInt16Range (detectorMode, 0, 3))
        return VI_ERROR_PARAMETER3;

	if (trace == 1)
    	Fmt (wrtBuf, "%s<%s", detTrcAModeArr[detectorMode]);
    else
    	Fmt (wrtBuf, "%s<%s", detTrcBModeArr[detectorMode]); 

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Trace Range                                                     */
/* Purpose:  This function sets number of measurement points.				 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confTraceRange (ViSession instrSession,
                ViBoolean measPoints)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;
    avr3267s_instrRange instrPtr;

    if (avr3267s_invalidViBooleanRange (measPoints))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", tracePointArr[measPoints]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    if (avr3267s_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))
        return avr3267s_status;
    
    if (instrPtr == NULL)                                             
        instrPtr = malloc (sizeof (struct avr3267s_statusDataRanges));

    if (measPoints == VI_ON)
    	instrPtr->traceRange = 1001;   
    else							   
    	instrPtr->traceRange = 501;

    if ((avr3267s_status = viSetAttribute (instrSession, VI_ATTR_USER_DATA, 
                            (ViUInt32)instrPtr)) < 0)
        return avr3267s_status;                                       


    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Marker Mode                                                     */
/* Purpose:  This function sets the Marker mode.							 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confMkrMode (ViSession instrSession, ViInt16 markerMode,
				ViReal64 markerFrequency, ViInt16 markerStep, ViReal64 stepSize)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;  
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (markerMode, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (markerFrequency, -26.5e9, 26.5e9))
        return VI_ERROR_PARAMETER3;
	if (avr3267s_invalidViBooleanRange (markerStep)) 
	    return VI_ERROR_PARAMETER4;                    
	if (avr3267s_invalidViReal64Range (stepSize, 0.0, 26.5e9))
	    return VI_ERROR_PARAMETER5;                                      

    if (markerMode == 0)
	    Fmt (wrtBuf, "%s<%s", mkrModeArr[markerMode]);
	else
		Fmt (wrtBuf, "%s<%s %fHZ", mkrModeArr[markerMode], markerFrequency);
	total += NumFmtdBytes();
	
	if (markerStep == VI_ON)
		Fmt (wrtBuf, "%s[a]<;MPA");
	else
		Fmt (wrtBuf, "%s[a]<;MPM %f", stepSize);
	total += NumFmtdBytes();

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Other Marker Functions                                          */
/* Purpose:  This function sets the Other Marker Functions.         		 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confOthMkrFunc (ViSession instrSession, ViInt16 markerFunctions)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (markerFunctions, 0, 5))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", othMkrArr[markerFunctions]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Signal Track Mode                                               */
/* Purpose:  This function sets the signal track mode. In this mode, the peak*/
/*           level of the signal on which a marker is displayed is detected  */
/*           on each sweep, and then the center frequency is moved to that   */
/*           frequency.														 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confSigTrack (ViSession instrSession, ViBoolean signalTrack)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViBooleanRange (signalTrack))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", sigTrackArr[signalTrack]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Marker Reference Object										 */
/* Purpose:  This function selects the reference object, when you would 	 */
/*           like to read relative level using normal marker.				 */
/*===========================================================================*/

ViStatus CVIFUNC avr3267s_confMkrRefObj (ViSession instrSession, ViInt16 referenceObject)
{
	ViUInt32 retCnt = 0;                        
	ViChar wrtBuf[BUFFER_SIZE];                 
	ViStatus avr3267s_status = VI_SUCCESS;      

	if (avr3267s_invalidViInt16Range (referenceObject, 0, 4))   
	    return VI_ERROR_PARAMETER2;
	    
	Fmt (wrtBuf, "%s<%s", refObjArr[referenceObject]);
	
	if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)  
	    return avr3267s_status;                                                            
	                                                                                       
	return avr3267s_status;                                                                
}

/*===========================================================================*/
/* Function: Printer                                                         */
/* Purpose:  This instrument can output the screen data to the printer that  */
/*           is equipped with the parallel interface based on the centronics */
/*           standards by using the PARALLEL port on the back panel. This	 */
/*           function sets specification of the output form.				 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confPrinter (ViSession instrSession, ViInt16 copyMode,
				ViInt16 printerCommand)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (copyMode, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViInt16Range (printerCommand, 0, 2))
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "%s<%s;%s", copyModeArr[copyMode], prnCommArr[printerCommand]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Bit Map File                                                    */
/* Purpose:  The screen data can be output to the memory card in the bit map */
/*           file format which is adapted by Microsoft Windows. This function*/
/*           sets specification of the data output.							 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_confBMP (ViSession instrSession, ViInt16 imageMode,
				ViBoolean RLECompression, ViInt16 fileNo)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (imageMode, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViBooleanRange (RLECompression))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViInt16Range (fileNo, 0, 999))
        return VI_ERROR_PARAMETER4;

    Fmt (wrtBuf, "%s<%s;%s;HCFILE %d[b2]", imgModeArr[imageMode], RLEArr[RLECompression],
    	fileNo);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Auto Manual Setup                                               */
/* Purpose:  This function selects auto mode or manual mode for some		 */
/*           contents, and sets value in manual mode.						 */
/*===========================================================================*/

ViStatus CVIFUNC avr3267s_confAutoManualSetup (ViSession instrSession,
                                               ViInt16 contents, ViBoolean mode,
                                               ViReal64 manualValue)
{
	ViUInt32 retCnt = 0;                      
    ViChar wrtBuf[BUFFER_SIZE];               
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (contents, 0, 4))     
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViBooleanRange (mode))
        return VI_ERROR_PARAMETER3;                     
    
    if (mode == VI_ON)
    	Fmt (wrtBuf, "%s<%s", setAutoArr[contents]);
    
    else
    	Fmt (wrtBuf, "%s<%s %f%s", setManualArr[contents], manualValue, 
    		setMnUnitArr[contents]);
    		
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)    
        return avr3267s_status;                                                              
                                                                                             
    return avr3267s_status;                                                                  
}

/*===========================================================================*/
/* Function: Mixer Setup													 */
/* Purpose:  This function sets the mixer parameters.						 */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_confMixerSetup (ViSession instrSession,
                                          ViBoolean mixer, ViInt16 bandN,
                                          ViBoolean bandLock, ViBoolean avgLoss,
                                          ViReal64 avgLossValue,
                                          ViBoolean lossvsFreq)
{
	ViUInt32 retCnt = 0;
	ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];                
    ViStatus avr3267s_status = VI_SUCCESS;     
    
    if (avr3267s_invalidViBooleanRange (mixer))         
        return VI_ERROR_PARAMETER2;                    
    if (avr3267s_invalidViInt16Range (bandN, 1, 14))
    	return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViBooleanRange (bandLock))         
        return VI_ERROR_PARAMETER4;                    
    if (avr3267s_invalidViBooleanRange (avgLoss))         
        return VI_ERROR_PARAMETER5;
    if (avr3267s_invalidViReal64Range (avgLossValue, -100.0, 100.0))    
        return VI_ERROR_PARAMETER6;
    if (avr3267s_invalidViBooleanRange (lossvsFreq))         
        return VI_ERROR_PARAMETER7;
    
    if (mixer==VI_ON)
    {
    	Fmt (wrtBuf, "%s<MXI");
    	total += NumFmtdBytes (); 
    }
    
    else
    {
    	Fmt (wrtBuf, "%s<MXE;BND %d;%s", bandN, bandLockArr[bandLock]);
    	total += NumFmtdBytes();
    	
    	if (avgLoss==VI_ON)
    		Fmt (wrtBuf, "%s[a]<;AGL ON %f", avgLossValue);
    	else
    		Fmt (wrtBuf, "%s[a]<;AGL OFF");
    	total += NumFmtdBytes();
    	
    	Fmt (wrtBuf, "%s[a]<;%s", lossvsFreqArr[lossvsFreq]);
    	total += NumFmtdBytes();	
    }
    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0)
        return avr3267s_status;                                                          
                                                                                         
    return avr3267s_status;                                                              
}

/*===========================================================================*/
/* Function: Mixer Bias Setup												 */
/* Purpose:  This function sets the external mixer's bias current.			 */
/*===========================================================================*/ 
ViStatus CVIFUNC avr3267s_confMixerBias (ViSession instrSession, ViReal64 mixerBiasValue)
{
	ViUInt32 retCnt = 0;                     
	ViChar wrtBuf[BUFFER_SIZE];              
	ViStatus avr3267s_status = VI_SUCCESS;   
	
	if (avr3267s_invalidViReal64Range (mixerBiasValue, -22.0, 22.0))
	    return VI_ERROR_PARAMETER2;                                 

	if (mixerBiasValue>=0.0)
		Fmt (wrtBuf, "%s<MXP %f", mixerBiasValue);
	else
		Fmt (wrtBuf, "%s<MXN %f", mixerBiasValue);

	if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avr3267s_status;                                                           
	                                                                                      
	return avr3267s_status;                                                               
}

/*===========================================================================*/
/* Function: Mixer Loss vs Frequency Table									 */
/* Purpose:  This function edits loss vs frequency table.					 */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_confMixerTable (ViSession instrSession,
                                          ViInt16 tableSize, ViReal64 frequency[],
                                          ViReal64 level[], ViReal64 bias[])
{
	ViUInt32 retCnt = 0;
	ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];             
    ViStatus avr3267s_status = VI_SUCCESS;
    ViInt16 n;
    ViReal64 *pt_freq, *pt_level, *pt_bias;
    
    if (avr3267s_invalidViInt16Range (tableSize, 0, 50)) 
        return VI_ERROR_PARAMETER2;
    n = tableSize;
    pt_freq = frequency;
    pt_level = level;
    pt_bias = bias;
    while (--n>=0)
    {
    	if (avr3267s_invalidViReal64Range (*pt_freq++, 10.0e9, 400.0e9))
    		return VI_ERROR_PARAMETER3;
    	if (avr3267s_invalidViReal64Range (*pt_level++, -50.0, 50.0)) 
    		return VI_ERROR_PARAMETER4;
    	if (avr3267s_invalidViReal64Range (*pt_bias++, -22.0, 22.0)) 
    		return VI_ERROR_PARAMETER5;                                  
    }
   
   	Fmt (wrtBuf, "%s<LVFDEL");
   	total += NumFmtdBytes();
   	
   	n = tableSize;             
   	pt_freq = frequency;       
    pt_level = level;              
    pt_bias = bias;
    while (--n>=0)
    {
    	Fmt (wrtBuf, "%s[a]<;LVFIN %fHZ,%fDB,%fMA", *pt_freq++, *pt_level++, 
    			*pt_bias++);
    	total += NumFmtdBytes();
    }
    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0)
        return avr3267s_status;                                                    
                                                                                   
    return avr3267s_status;                                                        
}

/*===========================================================================*/
/* Function: Sweep Action                                                    */
/* Purpose:  This function allows to perform Reset & Start Sweep or Single   */
/*           sweep action (Take Sweep).										 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actSweep (ViSession instrSession, ViInt16 sweepAction)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (sweepAction, 0, 3))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", sweepActionArr[sweepAction]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Auto of Coupled Functions                                       */
/* Purpose:  This function sets all of the coupled functions (Resolution	 */
/*           Bandwidth, Video Bandwidth and SWEEP TIME) to Auto based on the */
/*           current span setting.											 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actAutoCpl (ViSession instrSession)
{
    ViUInt32 retCnt = 0;
    ViStatus avr3267s_status = VI_SUCCESS;

    if ((avr3267s_status = viWrite (instrSession, "AL", 2, &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}


/*===========================================================================*/
/* Function: Trace A                                                         */
/* Purpose:  This function provides trace memory A operations.				 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actTraceA (ViSession instrSession, ViInt16 modesofTraceA,
				ViInt16 averagingValue)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (modesofTraceA, 0, 7))
        return VI_ERROR_PARAMETER2;
    if (modesofTraceA>=3 && modesofTraceA<=6)
    	if (avr3267s_invalidViInt16Range (averagingValue, 2, 999))
       		return VI_ERROR_PARAMETER3;

	if (modesofTraceA>=3 && modesofTraceA<=6)
		Fmt (wrtBuf, "%s<AG %d;%s", averagingValue, traceAArr[modesofTraceA]);
	else
		Fmt (wrtBuf, "%s<%s", traceAArr[modesofTraceA]);
	
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Trace B                                                         */
/* Purpose:  This function provides trace memory B operations.				 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actTraceB (ViSession instrSession, ViInt16 modesofTraceB)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (modesofTraceB, 0, 7))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", traceBArr[modesofTraceB]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Normalize A													 */
/* Purpose:  This function provides normalizing operations of trace A        */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_actNormalizeA (ViSession instrSession,
                                         ViBoolean normA, ViBoolean storeCorrData)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avr3267s_status = VI_SUCCESS;

	if (avr3267s_invalidViBooleanRange (normA))    
        return VI_ERROR_PARAMETER2;                         
    if (avr3267s_invalidViBooleanRange (storeCorrData))    
        return VI_ERROR_PARAMETER3;
    
    if (normA == VI_ON)
    {
    	if (storeCorrData == VI_ON)
    		Fmt (wrtBuf, "AR" );
    	else
    		Fmt (wrtBuf, "ANORM ON");
    		
    }
    
    else
    	Fmt (wrtBuf, "ANORM OFF");
    	
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)   	
        return avr3267s_status;                                                             	
                                                                                            	
    return avr3267s_status;                                                                 
}

/*===========================================================================*/ 
/* Function: Normalize B													 */ 
/* Purpose:  This function provides normalizing operations of trace B        */ 
/*===========================================================================*/ 
ViStatus CVIFUNC avr3267s_actNormalizeB (ViSession instrSession,
                                         ViBoolean normB, ViBoolean storeCorrData)
{
	ViUInt32 retCnt = 0;                                                                      
    ViChar wrtBuf[BUFFER_SIZE];                                                               
    ViStatus avr3267s_status = VI_SUCCESS;                                                    
                                                                                              
    if (avr3267s_invalidViBooleanRange (normB))                                               
        return VI_ERROR_PARAMETER2;                                                           
	if (avr3267s_invalidViBooleanRange (storeCorrData))                                       
        return VI_ERROR_PARAMETER3;                                                           
                                                                                              
    if (normB == VI_ON)                                                                       
    {                                                                                         
    	if (storeCorrData == VI_ON)                                                           
    		Fmt (wrtBuf, "BR" );                                                              
    	else                                                                                  
    		Fmt (wrtBuf, "BNORM ON");                                                         
    		                                                                                  
    }                                                                                         
                                                                                              
    else                                                                                      
    	Fmt (wrtBuf, "BNORM OFF");                                                            
    	                                                                                      
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)   	
        return avr3267s_status;                                                             	
                                                                                            	
    return avr3267s_status;                                                                   
}

/*===========================================================================*/
/* Function: Trace Exchange													 */
/* Purpose:  This function provides trace exchange between A and B           */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_actTraceExchange (ViSession instrSession)
{
	ViUInt32 retCnt = 0;                                                   
    ViStatus avr3267s_status = VI_SUCCESS;                                 
                                                                           
    if ((avr3267s_status = viWrite (instrSession, "ACHB", 4, &retCnt)) < 0)  
        return avr3267s_status;                                            
                                                                           
    return avr3267s_status;                                                
}

/*===========================================================================*/
/* Function: Display Line                                                    */
/* Purpose:  This function switchs displaying of limit line.				 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actDisplayLine (ViSession instrSession, ViBoolean displayLine,
                ViReal64 lineLevel)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViBooleanRange (displayLine))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (lineLevel, -240.0, 100.0))
        return VI_ERROR_PARAMETER3;

    if (lineLevel == VI_OFF)
    	Fmt (wrtBuf, "%s<%s", dispLineArr[displayLine]);
    else
    	Fmt (wrtBuf, "%s<%s %fDB", dispLineArr[displayLine], lineLevel);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/                             
/* Function: Limit Line                                                      */                             
/* Purpose:  This function sets limit line parameters.              		 */                             
/*===========================================================================*/                             
ViStatus _VI_FUNC CVIFUNC avr3267s_actLimitLine (ViSession instrSession,
										ViBoolean judge,
                                        ViBoolean limitLine1, ViBoolean line1PassRange,
                                        ViBoolean limitLine2, ViBoolean line2PassRange,
                                        ViBoolean xDataMode, ViInt16 xReference,
                                        ViReal64 xUserDefine, ViReal64 xOffset,
                                        ViBoolean yDataMode, ViInt16 yReference,
                                        ViReal64 yUserDefine, ViReal64 yOffset)                                
{
	ViUInt32 retCnt = 0;                  
	ViUInt32 total = 0;                   
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avr3267s_status = VI_SUCCESS;
	
	if (avr3267s_invalidViBooleanRange (judge))     
	    return VI_ERROR_PARAMETER2;                      
	if (avr3267s_invalidViBooleanRange (limitLine1))
	    return VI_ERROR_PARAMETER3;
	if (avr3267s_invalidViBooleanRange (line1PassRange))	    
	    return VI_ERROR_PARAMETER4;                      
	if (avr3267s_invalidViBooleanRange (limitLine2))	    
	    return VI_ERROR_PARAMETER5;                      
	if (avr3267s_invalidViBooleanRange (line2PassRange))	    
	    return VI_ERROR_PARAMETER6;                      
	if (avr3267s_invalidViBooleanRange (xDataMode))    
	    return VI_ERROR_PARAMETER7;              
	if (avr3267s_invalidViInt16Range (xReference, 0, 2))    
	    return VI_ERROR_PARAMETER8;                            
	if (avr3267s_invalidViReal64Range (xUserDefine, -100.0e9, 100.0e9))     
	    return VI_ERROR_INSTR_PARAMETER9;                                           
	if (avr3267s_invalidViReal64Range (xOffset, -100.0e9, 100.0e9))    
	    return VI_ERROR_INSTR_PARAMETER10;                                        
	if (avr3267s_invalidViBooleanRange (yDataMode))    
	    return VI_ERROR_INSTR_PARAMETER11;                      
	if (avr3267s_invalidViInt16Range (yReference, 0, 2))       
	    return VI_ERROR_INSTR_PARAMETER12;                            
	if (avr3267s_invalidViReal64Range (yUserDefine, -100.0, 100.0))      
	    return VI_ERROR_INSTR_PARAMETER13;                                          
	if (avr3267s_invalidViReal64Range (yOffset, -100.0, 100.0))          
	    return VI_ERROR_INSTR_PARAMETER14;                                          
	    
	Fmt (wrtBuf, "%s<%s;%s", PFJudgeArr[judge], limLine1Arr[limitLine1]);
	total += NumFmtdBytes ();
	if (limitLine1 == VI_ON)
	
	{
		Fmt (wrtBuf, "%s[a]<;%s", ln1PassRngArr[line1PassRange]);
		total += NumFmtdBytes ();
	}
	 
	Fmt (wrtBuf, "%s[a]<;%s", limLine2Arr[limitLine2]);                   
	total += NumFmtdBytes ();                                             
	if (limitLine2 == VI_ON)                                              
	{                                                                     
		Fmt (wrtBuf, "%s[a]<;%s", ln2PassRngArr[line2PassRange]);         
		total += NumFmtdBytes ();                                         
	}                                                                     
	    
	Fmt (wrtBuf, "%s[a]<;%s", xAxisArr[xDataMode]);
	total += NumFmtdBytes ();
	if (xDataMode == VI_OFF)
	{
		if (xReference == 2)
			Fmt (wrtBuf, "%s[a]<;%s %f", xRefArr[xReference], xUserDefine);
		else
			Fmt (wrtBuf, "%s[a]<;%s", xRefArr[xReference]);
		total += NumFmtdBytes ();		
	}
	
	Fmt (wrtBuf, "%s[a]<;LIMSFT %f", xOffset);
	total += NumFmtdBytes();
	    
	Fmt (wrtBuf, "%s[a]<;%s", yAxisArr[yDataMode]);                              
	total += NumFmtdBytes ();                                                    
	if (yDataMode == VI_OFF)                                                     
	{                                                                            
		if (yReference == 2)                                                     
			Fmt (wrtBuf, "%s[a]<;%s %f", yRefArr[yReference], yUserDefine);      
		else                                                                     
			Fmt (wrtBuf, "%s[a]<;%s", yRefArr[yReference]);                      
		total += NumFmtdBytes ();		                                         
	}                                                                            
	                                                                             
	Fmt (wrtBuf, "%s[a]<;LIMASFT %f", yOffset);                                   
	total += NumFmtdBytes();                   
	    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0)  
        return avr3267s_status;                                                            
                                                                                           
    return avr3267s_status;                                                                
}
                                                                                                           

/*===========================================================================*/
/* Function: Limit Line Table                                                */
/* Purpose:  This function performs limit line operations.					 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actLimLineTbl (ViSession instrSession, ViInt16 tableOperation,
                ViBoolean tableMode, ViReal64 frequencyValue, ViReal64 levelValue)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (tableOperation, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViBooleanRange (tableMode))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViReal64Range (frequencyValue, -400.0e9, 400.0e9))
        return VI_ERROR_PARAMETER4;
    if (avr3267s_invalidViReal64Range (levelValue, -240.0, 100.0))
        return VI_ERROR_PARAMETER5;

	switch (tableOperation) {
		case 0:
    		Fmt (wrtBuf, "%s<LIMTYP FREQ");
		break;
		case 1:
    		Fmt (wrtBuf, "%s<LIMTYP TIME");
		break;
		case 2:
			if (tableMode == VI_OFF) 
				Fmt (wrtBuf, "%s<LMTADEL");
			else
				Fmt (wrtBuf, "%s<LMTAIN %fHZ,%fDB", frequencyValue, levelValue);
		break;
		case 3:
			if (tableMode == VI_OFF) 
				Fmt (wrtBuf, "%s<LMTBDEL");
			else
				Fmt (wrtBuf, "%s<LMTBIN %fHZ,%fDB", frequencyValue, levelValue);
		break;
	}

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Limit Line Judgment                                             */
/* Purpose:  This function returns pass/fail judgment of limit line			 */
/*           operations. 													 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actLimLineJudg (ViSession instrSession,
				ViInt16 judgmentOperation, ViPInt16 judgmentResult,
				ViPInt16 noofFailPoints, ViReal64 _VI_FAR frequencyFailPoints[],
				ViReal64 _VI_FAR levelFailPoints[])
{
    ViUInt32 retCnt = 0;
    ViInt16 i = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (judgmentOperation, 0, 3))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", limitJudgArr[judgmentOperation]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;
    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avr3267s_status;

    if (judgmentOperation < 2) {
	    if (Scan (rdBuf, "%s>%d[b2]", judgmentResult) < 1)
    	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
	else {
	    if (Scan (rdBuf, "%s>%d[b2]", noofFailPoints) < 1)
    	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    	if (*noofFailPoints > 0)
    		for (i=0;i<*noofFailPoints;i++) {
			    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
			        return avr3267s_status;
			    if (Scan (rdBuf, "%s>%f,%f", &frequencyFailPoints[i], &levelFailPoints[i]) < 2)
    			    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    		}
    }

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Marker Data                                                     */
/* Purpose:  This function returns the current marker data (frequency and	 */
/*           level).														 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actMkrData (ViSession instrSession, ViPReal64 frequency,
				ViPReal64 level, ViPInt16 levelUnits)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if ((avr3267s_status = viWrite (instrSession, "MFL?", 4, &retCnt)) < 0)
        return avr3267s_status;
    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avr3267s_status;

    if (Scan (rdBuf, "%s>%f,%f", frequency, level) < 2)
   	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avr3267s_status = viWrite (instrSession, "AUNITS?", 7, &retCnt)) < 0)
        return avr3267s_status;
    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avr3267s_status;

    if (Scan (rdBuf, "%s>%d[b2]", levelUnits) < 1)
   	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Delta Marker Data												 */
/* Purpose:  This function returns the marker data (frequency and level).	 */
/*           Marker data can be selected from relative delta marker vs 		 */
/*           normal marker (Delta Marker), absolute normal marker (Marker 1) */
/*           and absolute delta marker (Marker 2).							 */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_actDltMkrData (ViSession instrSession,
                                         ViInt16 markerKind, ViReal64 *x,
                                         ViReal64 *level)
{
	ViUInt32 retCnt = 0;                                      
	ViChar wrtBuf[BUFFER_SIZE], rdBuf[BUFFER_SIZE];                                
	ViStatus avr3267s_status = VI_SUCCESS;
	
	if (avr3267s_invalidViInt16Range (markerKind, 0, 2))
		return VI_ERROR_PARAMETER2;

	Fmt (wrtBuf, "%s<%s", rdMkrXArr[markerKind]);
	
	if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)  
	    return avr3267s_status;                                                            
	if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)        
	    return avr3267s_status;
	    
	if (Scan (rdBuf, "%s>%f", x) < 1)          
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	
	Fmt (wrtBuf, "%s<%s", rdMkrLvlArr[markerKind]);                                         
	                                                                                      
	if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avr3267s_status;                                                           
	if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)       
	    return avr3267s_status;                                                           
	                                                                                      
	if (Scan (rdBuf, "%s>%f", level) < 1)                                                     
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;                                      
	
	return avr3267s_status;    
}

/*===========================================================================*/
/* Function: Peak Search                                                     */
/* Purpose:  This function sets the peak search actions. Finds the highest   */
/*           (lowest) level of the waveform being displayed. Displays that   */
/*           frequency and level.											 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actPeakSearch (ViSession instrSession, 
                ViInt16 peakSearchAction, ViBoolean contPeak)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (peakSearchAction, 0, 5))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViBooleanRange (contPeak))
        return VI_ERROR_PARAMETER3;
    
    Fmt (wrtBuf, "%s<%s;%s", peakActArr[peakSearchAction], contPeakArr[contPeak]);
    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Peak Search Condition                                           */
/* Purpose:  This functuin sets the peak search conditions.                  */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_actPkSchCond (ViSession instrSession,
                                        ViInt16 xAxisPeakRange,
                                        ViReal64 limitPosition,
                                        ViReal64 limitWidth, ViBoolean coupletoFT,
                                        ViInt16 yAxisPeakRange,
                                        ViReal64 peakDeltaY)
{
	ViUInt32 retCnt = 0;                   
	ViChar wrtBuf[BUFFER_SIZE];            
	ViStatus avr3267s_status = VI_SUCCESS;
	
	if (avr3267s_invalidViInt16Range (xAxisPeakRange, 0, 2))   
	    return VI_ERROR_PARAMETER2;
	if (avr3267s_invalidViReal64Range (limitPosition, 0.0, 26.5e9)) 
	    return VI_ERROR_PARAMETER3;                                     
	if (avr3267s_invalidViReal64Range (limitWidth, 0.0, 26.5e9))     
        return VI_ERROR_PARAMETER4;                                 
	if (avr3267s_invalidViBooleanRange (coupletoFT))
	    return VI_ERROR_PARAMETER5;                            
	if (avr3267s_invalidViInt16Range (yAxisPeakRange, 0, 8))          
	    return VI_ERROR_PARAMETER6;                                   
	if (avr3267s_invalidViReal64Range (peakDeltaY, 0.1, 10.0))   
	    return VI_ERROR_PARAMETER7;                                   

	Fmt (wrtBuf, "%s<%s;MKSPOS %fHZ;MKSWID %fHZ;%s;%s;DY %f", peakXRaArr[xAxisPeakRange],
		limitPosition, limitWidth, cpltoFTArr[coupletoFT],peakYRaArr[yAxisPeakRange],
		peakDeltaY);
	
	if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avr3267s_status;                                                           
	                                                                                      
	return avr3267s_status;                                                               
}

/*===========================================================================*/
/* Function: Multi Marker                                                    */
/* Purpose:  This function sets the multi-marker mode. With multi-marker	 */
/*           function, maximum 10 markers can be displayed. Each marker can  */
/*           be operate itself.												 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actMultiMarker (ViSession instrSession, ViInt16 multiMarkerMode,
                ViBoolean multiMarkerStatus, ViReal64 frequencyValue)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (multiMarkerMode, 0, 11))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViBooleanRange (multiMarkerStatus))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViReal64Range (frequencyValue, 0.0, 26.5e9))
        return VI_ERROR_PARAMETER4;

	switch (multiMarkerMode) {
		case 0:
	    	Fmt (wrtBuf, "%s<%s", multiMkrArr[multiMarkerStatus]);
		break;
		case 11:
			Fmt (wrtBuf, "%s<MN %fHZ", frequencyValue);
		break;
		default:
			if (multiMarkerStatus == VI_ON)
				Fmt (wrtBuf, "%s<%s %fHZ", multiMkrArr[multiMarkerMode+1], frequencyValue);
			else
				Fmt (wrtBuf, "%s<%s", multiMkrArr[multiMarkerMode+11]);
		break;
	}		

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Multi Marker Data                                               */
/* Purpose:  This function returns frequency and level values at multiple	 */
/*           points which are measured at the same time.					 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actMultiMkrData (ViSession instrSession,
				ViReal64 _VI_FAR frequency[], ViReal64 _VI_FAR level[],
				ViPInt16 levelUnits)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if ((avr3267s_status = viWrite (instrSession, "MLSF?", 5, &retCnt)) < 0)
        return avr3267s_status;
    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avr3267s_status;

    if (Scan (rdBuf, "%s>%11f[x]", frequency) < 1)
   	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avr3267s_status = viWrite (instrSession, "MLSL?", 5, &retCnt)) < 0)
        return avr3267s_status;
    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avr3267s_status;

    if (Scan (rdBuf, "%s>%11f[x]", level) < 1)
   	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if ((avr3267s_status = viWrite (instrSession, "AUNITS?", 7, &retCnt)) < 0)
        return avr3267s_status;
    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avr3267s_status;

    if (Scan (rdBuf, "%s>%d[b2]", levelUnits) < 1)
   	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Peak List                                                       */
/* Purpose:  This function lists up frequency and level of all marker		 */
/*           positions.														 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actPeakList (ViSession instrSession,
				ViInt16 peakListMode, ViReal64 _VI_FAR frequency[],
				ViReal64 _VI_FAR level[], ViPInt16 levelUnits)
{
    ViUInt32 retCnt = 0;
    ViInt16 i, n;
    ViChar wrtBuf[BUFFER_SIZE];
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (peakListMode, 0, 3))
        return VI_ERROR_PARAMETER2;

    if (peakListMode == 3) {
	    if ((avr3267s_status = viWrite (instrSession, "PKLST?", 6, &retCnt)) < 0)
    	    return avr3267s_status;
	    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
    	    return avr3267s_status;
	    if (Scan (rdBuf, "%s>%d[b2]", &n) < 1)
   		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   		for (i=0;i<n;i++) {
   			if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   			    return avr3267s_status;                                                    
		    if (Scan (rdBuf, "%s>%f,%f", &frequency[i], &level[i]) < 2)
   			    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   		}

	    if ((avr3267s_status = viWrite (instrSession, "AUNITS?", 7, &retCnt)) < 0)
    	    return avr3267s_status;
    	if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
	        return avr3267s_status;
	    if (Scan (rdBuf, "%s>%d[b2]", levelUnits) < 1)
   		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    }
    else {
	    Fmt (wrtBuf, "%s<%s", peakListArr[peakListMode]);
    	if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        	return avr3267s_status;
    }
    
    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Marker To                                                       */
/* Purpose:  This function sets the current marker data (frequency, level,   */
/*           delta, etc.) as the data for some other function.				 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actMarkerTo (ViSession instrSession, ViInt16 marker)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (marker, 0, 9))
        return VI_ERROR_PARAMETER2;
    
	Fmt (wrtBuf, "%s<%s", markerToArr[marker]);
	
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: X dB Down                                                       */
/* Purpose:  This function displays the frequency or level difference between*/
/*           the reference marker and a marker displayed at X dB lower (or   */
/*           higher) position, and moves marker to desired position.		 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actdBDown (ViSession instrSession, ViInt16 dBDown,
                ViReal64 dBDownWidth)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (dBDown, 0, 9))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (dBDownWidth, -100.0, 100.0))
        return VI_ERROR_PARAMETER3;

    if (dBDown == 0) {
	    Fmt (wrtBuf, "%s<%s %fDB", dBDownArr[dBDown], dBDownWidth);
    }
    else {
        Fmt (wrtBuf, "%s<%s", dBDownArr[dBDown]);
    }
    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Execute 3rd Order Measurement									 */
/* Purpose:  When this function is executed, delta marker is displayed on    */
/*           the peak of fundamental wave,and normal marker is displayed on  */
/*           the peak of third order intermodulation distiortion.			 */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_act3rdOrder (ViSession instrSession)
{
	ViUInt32 retCnt = 0;                     
    ViChar wrtBuf[BUFFER_SIZE];              
    ViStatus avr3267s_status = VI_SUCCESS;
    
    Fmt (wrtBuf, "%s<PKTHIRD");
    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;                                                          
                                                                                         
    return avr3267s_status;                                                              
}


/*===========================================================================*/
/* Function: % AM Modulation Measurement                                     */
/* Purpose:  This function selects AM modulation measurement.				 */
/*===========================================================================*/

ViStatus CVIFUNC avr3267s_actAMMdl (ViSession instrSession, ViInt16 modAMMeas)
{
	ViUInt32 retCnt = 0;                     
    ViChar wrtBuf[BUFFER_SIZE];              
    ViStatus avr3267s_status = VI_SUCCESS;
    
    Fmt (wrtBuf, "%s<%s", modAMArr[modAMMeas]);
    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;                                                          
                                                                                         
    return avr3267s_status;                                                              
}

/*===========================================================================*/
/* Function: Measurement Window                                              */
/* Purpose:  This function sets window display conditions.   				 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actMeasWindow (ViSession instrSession,
				ViBoolean windowState, ViReal64 centerPosition,
				ViReal64 windowWidth, ViBoolean windowSweep)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViBooleanRange (windowState))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (centerPosition, 0.0, 26.5e9))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViReal64Range (windowWidth, 0.0, 26.5e9))
        return VI_ERROR_PARAMETER4;
    if (avr3267s_invalidViBooleanRange (windowSweep))
        return VI_ERROR_PARAMETER5;

    Fmt (wrtBuf, "%s<WLX %fHZ;WDX %fHZ;%s;%s", centerPosition, windowWidth,
    	windStateArr[windowState], windSweepArr[windowSweep]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}
                                          
/*===========================================================================*/
/* Function: Multi-Screen                                                    */
/* Purpose:  This function sets multi-screen display conditions.             */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actMultiScreen (ViSession instrSession,
				ViInt16 multiScreenMode, ViReal64 centerPosition,
				ViReal64 windowWidth, ViInt16 peakZoom, ViBoolean screenSide)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (multiScreenMode, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (centerPosition, 0.0, 26.5e9))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViReal64Range (windowWidth, 0.0, 26.5e9))
        return VI_ERROR_PARAMETER4;
    if (avr3267s_invalidViInt16Range (peakZoom, 0, 3))             
        return VI_ERROR_PARAMETER2;                                       
    if (avr3267s_invalidViBooleanRange (screenSide))
        return VI_ERROR_PARAMETER5;

    Fmt (wrtBuf, "%s<%s;ZMPOS %fHZ;ZMWID %fHZ;%s;%s", multiScrArr[multiScreenMode],
    	centerPosition, windowWidth, pkZmArr[peakZoom], scrSideArr[screenSide]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Input Corr Factor                                               */
/* Purpose:  The correction factor function corrects the level in the		 */
/*           specified frequency range.										 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actInputCorr (ViSession instrSession,
				ViInt16 correctionMode, ViReal64 frequencyCorrData, 
				ViReal64 levelCorrData)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (correctionMode, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (frequencyCorrData, 0.0, 8.0e9))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViReal64Range (levelCorrData, -50.0, 50.0))
        return VI_ERROR_PARAMETER4;

    if (correctionMode == 2) {
    	Fmt (wrtBuf, "%s<CRIN %fHZ,%fDB", frequencyCorrData, levelCorrData);
    }
    else {
    	Fmt (wrtBuf, "%s<%s", levelCorrArr[correctionMode]);
    }
    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Hard Copy Control                                               */
/* Purpose:  This function sets the target device of the screen data and	 */
/*           execute hard copy operation. Printer, plotter and memory card   */
/*           can be selected for the target device of the screen data output.*/
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actHardCopy (ViSession instrSession, ViInt16 deviceSelection)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (deviceSelection, 0, 3))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s;HCOPY", deviceSelArr[deviceSelection]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Calibration                                                     */
/* Purpose:  This function sets and executes the calibration routines.		 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actCalibration (ViSession instrSession,
				ViInt16 calibrationFunction, ViReal64 calibrationLevel,
				ViInt16 correctValue, ViBoolean freqCorr,
				ViBoolean calCorr)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (calibrationFunction, 0, 10))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (calibrationLevel, -36.0, -10.0))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViInt16Range (correctValue, 0, 255))
        return VI_ERROR_PARAMETER4;
    if (avr3267s_invalidViBooleanRange (freqCorr))
        return VI_ERROR_PARAMETER5;
    if (avr3267s_invalidViBooleanRange (calCorr))
        return VI_ERROR_PARAMETER6;

	Fmt (wrtBuf, "%s<%s;%s;%s", freqCorrArr[freqCorr],
		calCorrArr[calCorr], calFuncArr[calibrationFunction]);
    total += NumFmtdBytes();

	switch (calibrationFunction) {
		case 8:
    		Fmt (wrtBuf, "%s[a]< %fDB", calibrationLevel);
    		total += NumFmtdBytes();
		break;
		case 9:
    		Fmt (wrtBuf, "%s[a]< %d", correctValue);
    		total += NumFmtdBytes();
		break;
		case 10:                                                   
			Fmt (wrtBuf, "%s[a]< %d", correctValue);              
			total += NumFmtdBytes();                              
		break;                                                    
	}

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Label                                                           */
/* Purpose:  This function sets label input. Label input is performed for the*/
/*           waveform display. The documentation text can be used for a		 */
/*           plotter output and a memory card function.						 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actLabel (ViSession instrSession,
				ViBoolean labelStaus, ViChar _VI_FAR labelString[])
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViBooleanRange (labelStaus))
        return VI_ERROR_PARAMETER2;
    if (StringLength (labelString) > 30)
        return VI_ERROR_PARAMETER3;

    if (labelStaus == VI_ON) {
    	Fmt (wrtBuf, "%s<%s %s", labelArr[labelStaus], labelString);
    }
    else {
    	Fmt (wrtBuf, "%s<%s", labelArr[labelStaus]);
    }
    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Power Meas                                                      */
/* Purpose:  This function selects power measurement mode.					 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actPowerMeas (ViSession instrSession, ViInt16 powerMeasMode,
				ViInt16 averageTime)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (powerMeasMode, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViInt16Range (averageTime, 1, 999))
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "%s<%s;PWTM %d[b2]", powMeasArr[powerMeasMode], averageTime);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Counter                                                         */
/* Purpose:  This function sets the counter mode and resolution of frequency */
/*           counter.														 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actCounter (ViSession instrSession, ViBoolean counterMode,
				ViInt16 resolution)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViBooleanRange (counterMode))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViInt16Range (resolution, 0, 3))
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "%s<%s;%s", cntResArr[resolution], cntModeArr[counterMode]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Sound                                                           */
/* Purpose:  This function sets and activates the sound mode parameters.	 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actSound (ViSession instrSession, ViInt16 soundMode,
				ViInt16 volume, ViReal64 pauseTime, ViBoolean squelchMode,
				ViReal64 squelchValue)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (soundMode, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViInt16Range (volume, 1, 8))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViReal64Range (pauseTime, 0.1, 1000.0))
        return VI_ERROR_PARAMETER4;
    if (avr3267s_invalidViBooleanRange (squelchMode))
        return VI_ERROR_PARAMETER5;

    Fmt (wrtBuf, "%s<PU %f;SQE %f;%s;SDV %d[b2];%s", pauseTime, squelchValue,
    	squelchArr[squelchMode], volume, soundModeArr[soundMode]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Noise/Hz                                                        */
/* Purpose:  This function sets the Noise/Hz measurement.					 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actNoiseHz (ViSession instrSession, ViInt16 noiseHzMode,
                ViReal64 noiseFrq)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (noiseHzMode, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViReal64Range (noiseFrq, 1.0, 100.0e6))
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "%s<NI %f;%s", noiseFrq, noiseModeArr[noiseHzMode]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: OBW                                                             */
/* Purpose:  This function sets the OBW (Occupied Bandwidth) parameters and  */
/*           calculates the occupied bandwidth of the waveform displayed on  */
/*           the screen. For the result call OBW Data function.				 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actOBW (ViSession instrSession, ViInt16 setup, ViReal64 OBW, 
			ViInt16 averageTime)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

	if (avr3267s_invalidViInt16Range (setup, 0, 2))          
	    return VI_ERROR_PARAMETER3;                                  
	if (avr3267s_invalidViReal64Range (OBW, 10.0, 99.8))    
	    return VI_ERROR_PARAMETER2;                         
    if (avr3267s_invalidViInt16Range (averageTime, 1, 999))
        return VI_ERROR_PARAMETER3;
    
    

    Fmt (wrtBuf, "%s<AVGOBW %d[b2];OBWST %s;OBWPER %f;OBW ON", averageTime, setUpArr[setup], OBW);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: ACP                                                             */
/* Purpose:  This function sets ACP (Adjacent Channels Leakage Power)		 */
/*           parameters and calculates the total power from measured data	 */
/*           displayed on the screen, and integrates power in specified		 */
/*           bandwidth to obtain the ratio to the total power.				 */
/*===========================================================================*/
ViStatus _VI_FUNC CVIFUNC avr3267s_actACP (ViSession instrSession, ViInt16 ACPSetUp,
                                  ViBoolean ACPScreen, ViBoolean ACPGraph,
                                  ViReal64 carrierbandWidth, ViInt16 ACPNumber,
                                  ViReal64 ACPCSHz[], ViReal64 ACPBSHz[],
                                  ViInt16 averageTime)
{
	ViUInt32 retCnt = 0;
	ViUInt32 total =0;
	ViChar wrtBuf[BUFFER_SIZE];                                   
	ViStatus avr3267s_status = VI_SUCCESS;
	ViInt16 n;
	ViReal64 *pt_CS, *pt_BS;
	                                                              		
	if (avr3267s_invalidViInt16Range (ACPSetUp, 0, 2))            
	    return VI_ERROR_PARAMETER2;                               
	if (avr3267s_invalidViBooleanRange (ACPScreen))               
	    return VI_ERROR_PARAMETER3;                               
	if (avr3267s_invalidViBooleanRange (ACPGraph))                
	    return VI_ERROR_PARAMETER4;
	if (avr3267s_invalidViReal64Range (carrierbandWidth, 0.0, 100.0e9))     
		return VI_ERROR_PARAMETER5;                                 
	if (avr3267s_invalidViInt16Range (ACPNumber, 1, 5))      
	    return VI_ERROR_PARAMETER6;
	n = ACPNumber;
	pt_CS = ACPCSHz;
	pt_BS = ACPBSHz;
	while (--n>=0)
	{
		if (avr3267s_invalidViReal64Range (*pt_CS++, 0.0, 100.0e9))
			return VI_ERROR_PARAMETER7;
		if (avr3267s_invalidViReal64Range (*pt_BS++, 0.0, 100.0e9))   
			return VI_ERROR_PARAMETER8; 		                      
	}
	if (avr3267s_invalidViInt16Range (averageTime, 1, 999))
	    return VI_ERROR_INSTR_PARAMETER9;
	
	Fmt (wrtBuf, "%s<ACPST %s;%s;%s", setUpArr[ACPSetUp], screenArr[ACPScreen], 
		graphArr[ACPGraph]); 
    total += NumFmtdBytes();
    
    if (ACPGraph == VI_OFF)
    {
    	Fmt (wrtBuf, "%s[a]<;ACPAVG %d;CARRBS %fHZ", averageTime, carrierbandWidth);
    	total += NumFmtdBytes();
    }
    
    n = ACPNumber;        
    pt_CS = ACPCSHz;      
    pt_BS = ACPBSHz;      
    Fmt (wrtBuf, "%s[a]<;CSBSDEL");
    total += NumFmtdBytes();
    while (--n>=0)
    {
    	Fmt (wrtBuf, "%s[a]<;CSBSIN %fHZ,%fHZ", *pt_CS++, *pt_BS++);
    	total += NumFmtdBytes();
    }
    
    Fmt (wrtBuf, "%s[a]<;ACP ON");
    total += NumFmtdBytes();
    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0) 
        return avr3267s_status;                                                           
                                                                                          
    return avr3267s_status;                                                               
}

/*===========================================================================*/
/* Function: ACP Filter                                                      */
/* Purpose:  This function sets the parameters of the root nyquist filter	 */
/*           which is used in ACP measurement.								 */
/*===========================================================================*/

ViStatus CVIFUNC avr3267s_actACPFilter (ViSession instrSession, ViInt16 filterControl,
                                        ViReal64 symbolRate, ViReal64 rolloffFactor)
{
	ViUInt32 retCnt = 0;
	ViChar wrtBuf[BUFFER_SIZE];                
	ViStatus avr3267s_status = VI_SUCCESS;
	
	if (avr3267s_invalidViInt16Range (filterControl, 0, 2))             
	    return VI_ERROR_PARAMETER2;
	if (avr3267s_invalidViReal64Range (symbolRate, 1.0, 1.0e9))  
		return VI_ERROR_PARAMETER3;
	if (avr3267s_invalidViReal64Range (rolloffFactor, 0.01, 0.99))   
		return VI_ERROR_PARAMETER3;                               	
		
	switch (filterControl)
	{
	case 0:
		Fmt (wrtBuf, "%s<SYMRT %fHZ;RFACT %f", symbolRate, rolloffFactor);
		break;
		
	case 1:
		Fmt (wrtBuf, "%s<NQST OFF");
		break;
		
	case 2:
		Fmt (wrtBuf, "%s<NQST ON");
		break;
	}
	
	if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)  
	    return avr3267s_status;                                                  	
	                                                                             	
	return avr3267s_status;                                                      	
}

/*===========================================================================*/
/* Function: HARM                                                            */
/* Purpose:  This function starts higher harmonics measuring mode.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actHARM (ViSession instrSession, ViBoolean slctFund, 
									ViReal64 fundFreq, ViInt16 harmonicNo)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;
    avr3267s_instrRange instrPtr;
    
    if (avr3267s_invalidViBooleanRange (slctFund))
        return VI_ERROR_PARAMETER2;                
    if (avr3267s_invalidViReal64Range (fundFreq, 1.0e3, 4.0e9))
        return VI_ERROR_PARAMETER3;
    if (avr3267s_invalidViInt16Range (harmonicNo, 2, 10))
        return VI_ERROR_PARAMETER4;

	if (slctFund==VI_ON)
    	Fmt (wrtBuf, "%s<HRMFND ON;HRMFND %fHZ;HARM ON", fundFreq);
	else
		Fmt (wrtBuf, "%s<HRMFND OFF");
	total += NumFmtdBytes();
	
	Fmt (wrtBuf, "%s[a]<;HRMNUM %d[b2];HARM ON", harmonicNo);
	total += NumFmtdBytes();

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0)
        return avr3267s_status;

    if (avr3267s_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))
        return avr3267s_status;
    
    if (instrPtr == NULL)                                                    
        instrPtr = malloc (sizeof (struct avr3267s_statusDataRanges));       

   	instrPtr -> harmonics = harmonicNo;

    if ((avr3267s_status = viSetAttribute (instrSession, VI_ATTR_USER_DATA, 
                            (ViUInt32)instrPtr)) < 0)
        return avr3267s_status;                                       

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Spurious Measurement                                            */
/* Purpose:  This function switches spurious measurement.                    */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_actSprMeas (ViSession instrSession, ViBoolean sprMeasMode)
{
	ViUInt32 retCnt = 0;                         
	ViChar wrtBuf[BUFFER_SIZE];                  
	ViStatus avr3267s_status = VI_SUCCESS;       

	if (avr3267s_invalidViBooleanRange (sprMeasMode))          
        return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuf, "%s<%s", sprMeasArr[sprMeasMode]);
    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes(), &retCnt)) < 0) 
        return avr3267s_status;                                                  	     
   
    return avr3267s_status;                                                     
}

ViStatus CVIFUNC avr3267s_actSprTbl (ViSession instrSession, ViInt16 tableNo,
                                     ViBoolean loadTable, ViBoolean saveTable,
                                     ViInt16 tableSize,
                                     ViReal64 startFrequency[],
                                     ViReal64 stopFrequency[], ViReal64 RBW[],
                                     ViReal64 limitLevel[])
{
	ViUInt32 retCnt = 0;
	ViUInt32 total = 0;
	ViChar wrtBuf[BUFFER_SIZE];                  
	ViStatus avr3267s_status = VI_SUCCESS;
	ViInt16 i, n;
	ViReal64 *pt_start, *pt_stop, *pt_RBW, *pt_limit;
	
	if (avr3267s_invalidViInt16Range (tableNo, 1, 3))
		return VI_ERROR_PARAMETER2;
	if (avr3267s_invalidViBooleanRange (loadTable))   
	    return VI_ERROR_PARAMETER3;                   
	if (avr3267s_invalidViBooleanRange (saveTable))    
	    return VI_ERROR_PARAMETER4;                   
	if (avr3267s_invalidViInt16Range (tableSize, 0, 15))   
		return VI_ERROR_PARAMETER5;                    

	n = tableSize;
	pt_start = startFrequency;
	pt_stop = stopFrequency;
	pt_RBW = RBW;
	pt_limit = limitLevel;
	while (--n>=0)
	{
		if (avr3267s_invalidViReal64Range (*pt_start++, 0.0, 26.5e9)) 
			return VI_ERROR_PARAMETER6;                             
		if (avr3267s_invalidViReal64Range (*pt_stop++, 0.0, 26.5e9))       
			return VI_ERROR_PARAMETER7;                                     
		if (avr3267s_invalidViReal64Range (*pt_RBW++, 10.0, 10.0e6))       
			return VI_ERROR_PARAMETER8;                                     
		if (avr3267s_invalidViReal64Range (*pt_limit++, -100.0, 100.0))       
			return VI_ERROR_INSTR_PARAMETER9;                                     
	}

	Fmt (wrtBuf, "%s<SPRTBL %d", tableNo);
	total += NumFmtdBytes();
	
	if (loadTable==VI_ON)
	{
		Fmt (wrtBuf, "%s[a]<;SPRLD");
		total += NumFmtdBytes();
	}
	
	else
	{
		Fmt( wrtBuf, "%s[a]<;SPRDEL");
		total += NumFmtdBytes();
		
		n = tableSize;                    
		pt_start = startFrequency;        
		pt_stop = stopFrequency;          
		pt_RBW = RBW;                     
		pt_limit = limitLevel;
		for (i=1; i<=n; i++)
		{
			Fmt (wrtBuf, "%s[a]<;SPRIN %d, %fHZ,%fHz,%fHZ,%fDB", i, *pt_start++, *pt_stop++,
					*pt_RBW++, *pt_limit++);
			total += NumFmtdBytes();
		}
		
		if (saveTable==VI_ON)
		{
			Fmt (wrtBuf, "%s[a]<;SPRSV");
			total += NumFmtdBytes();
		}
	}
	
	if ((avr3267s_status = viWrite (instrSession, wrtBuf, total, &retCnt)) < 0)   
	    return avr3267s_status;                                                  	       
	                                                                                       
	return avr3267s_status;                                                                
}

/*===========================================================================*/
/* Function: Artificial ANALOG												 */
/* Purpose:  This function provides artificial analog display controls.		 */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_actArtificialAnalog (ViSession instrSession,
                                               ViBoolean artificialAnalog,
                                               ViBoolean traceDisplay,
                                               ViInt16 samplingTimes)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avr3267s_status = VI_SUCCESS;

	if (avr3267s_invalidViBooleanRange (artificialAnalog))     
	    return VI_ERROR_PARAMETER2;                        
	if (avr3267s_invalidViBooleanRange (traceDisplay))     
	    return VI_ERROR_PARAMETER3;                        
	if (avr3267s_invalidViInt16Range (samplingTimes, 2, 32))   
	    return VI_ERROR_PARAMETER4;                               

	if (artificialAnalog==VI_ON)
	{
		if (traceDisplay==VI_ON)
			Fmt (wrtBuf, "%s<ANLG ON;ANLGDSP PAUS");
		
		else
			Fmt (wrtBuf, "%s<ANLG ON;ANLGDSP CONT;ANLGTM %d", samplingTimes);
	}
	
	else
		Fmt (wrtBuf, "%s<ANLG OFF");

	if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)  
	    return avr3267s_status;                                                  	      
	                                                                                      
	return avr3267s_status;                                                               
}

/*===========================================================================*/
/* Function: XY Cursor														 */
/* Purpose:  This function provides XY cursor controls.						 */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_actXYCursor (ViSession instrSession,
                                       ViBoolean XYCursor, ViInt16 anchor,
                                       ViReal64 XCursorPosition,
                                       ViReal64 YCursorPosition)
{
	ViUInt32 retCnt = 0;                               
    ViChar wrtBuf[BUFFER_SIZE];                        
    ViStatus avr3267s_status = VI_SUCCESS;             
    
    if (avr3267s_invalidViBooleanRange (XYCursor))      
        return VI_ERROR_PARAMETER2;                         
    if (avr3267s_invalidViInt16Range (anchor, 0, 2))
        return VI_ERROR_PARAMETER3;                         
    if (avr3267s_invalidViReal64Range (XCursorPosition, -26.5e9, 26.5e9))
    	return VI_ERROR_PARAMETER4;
    
    if (XYCursor==VI_ON)
    	Fmt (wrtBuf, "%s<XYCSR ON;CSRX %f;CSRY %f;%s", XCursorPosition,
    			YCursorPosition, XYAnchorArr[anchor]);
    
    else
    	Fmt (wrtBuf, "%s<XYCSR OFF");
    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)   
        return avr3267s_status;                                                  	       
                                                                                           
    return avr3267s_status;                                                                
}

/*===========================================================================*/
/* Function: Standard Operation Status Reg                                   */
/* Purpose:  This function supports the standard operation status register.  */
/*           The event register for the standard operation status is used to */
/*           hold the change from 1 to 0 of the corresponding condition		 */
/*           register. Event register has the enable register to determine   */
/*           which bit to be available. The enable register sets the		 */
/*           corresponding bit in decimal value (OPR bit of the status byte  */
/*           register is set in 1 when desired conditions are accomplished). */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actStdOSR (ViSession instrSession,
				ViBoolean registerMode, ViInt32 registerValue,
                ViPInt32 registerStatus)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViBooleanRange (registerMode))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViInt32Range (registerValue, 0, 65535))
        return VI_ERROR_PARAMETER3;

	if (registerMode == VI_ON) {
    	Fmt (wrtBuf, "%s<OPR %d[b4]", registerValue);
	    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
    	    return avr3267s_status;
	}
	else {
	    if ((avr3267s_status = viWrite (instrSession, "OPREVT?", 7, &retCnt)) < 0)
    	    return avr3267s_status;
	    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
    	    return avr3267s_status;
   		if (Scan (rdBuf, "%s>%d[b4]", registerStatus) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
	
    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Standard Event Register                                         */
/* Purpose:  This function supports the event register operations. The event */
/*           register latches and and retains the status information from the*/
/*           condition register. Once the register is set, the condition is  */
/*           maintained until a query command reads out the information or   */
/*           the register is reset.											 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actStdER (ViSession instrSession,
                ViInt16 registerMode, ViInt16 registerValue,
                ViPInt16 registerStatus)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (registerMode, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViInt16Range (registerValue, 0, 256))
        return VI_ERROR_PARAMETER3;

	switch (registerMode) {
		case 0:
	    	Fmt (wrtBuf, "%s<*ESE %d[b2]", registerValue);
		    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
    		    return avr3267s_status;
    	break;
    	case 1:
		    if ((avr3267s_status = viWrite (instrSession, "*ESR?", 5, &retCnt)) < 0)
    		    return avr3267s_status;
	    	if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
    	    	return avr3267s_status;
   			if (Scan (rdBuf, "%s>%d[b2]", registerStatus) < 1)
	    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    	break;
    	case 2:
		    if ((avr3267s_status = viWrite (instrSession, "*CLS", 4, &retCnt)) < 0)
    		    return avr3267s_status;
    	break;
	}

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Status Byte Register                                            */
/* Purpose:  The status byte register sumarizes the information from the	 */
/*           status register. In addition, a summary of the status byte		 */
/*           register is sent to the controller as a service request.		 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_actSTB (ViSession instrSession,
                ViBoolean registerMode, ViInt16 registerValue,
                ViPInt16 registerStatus)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViBooleanRange (registerMode))
        return VI_ERROR_PARAMETER2;
    if (avr3267s_invalidViInt16Range (registerValue, 0, 256))
        return VI_ERROR_PARAMETER3;

	if (registerMode == VI_ON) {
    	Fmt (wrtBuf, "%s<*SRE %d[b2]", registerValue);
	    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
    	    return avr3267s_status;
	}
	else {
	    if ((avr3267s_status = viWrite (instrSession, "*STB?", 5, &retCnt)) < 0)
    	    return avr3267s_status;
	    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
    	    return avr3267s_status;
   		if (Scan (rdBuf, "%s>%d[b2]", registerStatus) < 1)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
	
    return avr3267s_status;
}


/*===========================================================================*/
/* Function: Service Request												 */
/* Purpose:  This function provides service request controls.				 */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_actServiceRequest (ViSession instrSession,
                                             ViBoolean serviceRequest, ViInt16 dataSRE)
{
	ViUInt32 retCnt = 0;                                      
	ViChar wrtBuf[BUFFER_SIZE];                               
	ViStatus avr3267s_status = VI_SUCCESS;                    
	                                                          
	if (avr3267s_invalidViBooleanRange (serviceRequest))        
	    return VI_ERROR_PARAMETER2;                           
	if (avr3267s_invalidViInt16Range (dataSRE, 0, 255)) 
	    return VI_ERROR_PARAMETER3;                           

	Fmt (wrtBuf, "%s<%s;*SRE %d", ServiceReqArr[serviceRequest], dataSRE);
	
	if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)  
	    return avr3267s_status;                                                            
	                                                                                       
	return avr3267s_status;                                                                
}

/*===========================================================================*/
/* Function: Set Mask Bit and Clear STB										 */
/* Purpose:  This function sets standard operation resister and standard	 */
/*           event resister, and clear status byte.							 */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_actSetMaskBit_ClrSTB (ViSession instrSession,
                                                ViInt32 oprMaskBit,
                                                ViInt32 eventMaskBit)
{
	ViUInt32 retCnt = 0;                  
	ViChar wrtBuf[BUFFER_SIZE];           
	ViStatus avr3267s_status = VI_SUCCESS;
	
	if (avr3267s_invalidViInt32Range (oprMaskBit, 0, 65535))  
	    return VI_ERROR_PARAMETER2;                      
	if (avr3267s_invalidViInt32Range (eventMaskBit, 0, 255))  
	    return VI_ERROR_PARAMETER3;                      

	Fmt (wrtBuf, "%s<OPR %d;*ESE %d;*CLS", oprMaskBit, eventMaskBit);

	if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0) 
	    return avr3267s_status;                                                           
	                                                                                      
    return avr3267s_status;                                                               
}

/*===========================================================================*/
/* Function: Wait for Meas													 */
/* Purpose:  From measurement started, this function checks that measurement */
/*           stopps, or measutement time reaches setting maximum waiting	 */
/*           time.															 */
/*===========================================================================*/

ViInt32 strng_sec(char *tm)                   
{                                             
	int a,b,c;                                
	ViInt32 vSec;                             
	                                          
	Scan (tm, "%s>%d:%d:%d", &a, &b, &c);         
	vSec = a*3600 + b*60 + c;                 
	return vSec;                              
}                                             

ViStatus CVIFUNC avr3267s_actWaitMeas (ViSession instrSession,
                                       ViInt32 maxWaitTime, ViBoolean *errorFlag)
{
	ViUInt32 retCnt = 0;                  
    ViChar wrtBuf[BUFFER_SIZE], rdBuf[BUFFER_SIZE];           
    ViStatus avr3267s_status = VI_SUCCESS;
    ViInt16 flag = 0;
    ViInt16 status;
    ViInt32 strSec, stpSec, realSec;
    
    if (avr3267s_invalidViInt32Range (maxWaitTime, 0, 3600))           
        return VI_ERROR_PARAMETER2;                        
    
	strSec = strng_sec (TimeStr());
    stpSec = strSec + maxWaitTime; 
    
    while (flag==0)
    {
    	if ((avr3267s_status = viWrite (instrSession, "*STB?", 5, &retCnt)) < 0)       
    	    return avr3267s_status;                                                    
    	if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
    	    return avr3267s_status;                                                    
    	if (Scan (rdBuf, "%s>%d[b2]", &status) < 1)                             
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;                               
    
    	realSec = strng_sec (TimeStr());
    	if (realSec < strSec)
    		realSec += 86400;
    	
    	if (status>=128 || realSec>stpSec)
    		flag = 1;
    	
		Delay (2.0e-3);
    	
    }
    
    if (status == 128)
    	*errorFlag = VI_OFF;
    else
    	*errorFlag = VI_ON;
    
	return avr3267s_status;    
}

/*===========================================================================*/
/* Function: Trace Data Input                                                */
/* Purpose:  This function sets trace data array. Trace data on the screen   */
/*           includes the data for 1001 or 501 points on frequency axis. For */
/*           inputting these data, it is necessary to transfer data for		 */
/*           1001/501 points from leftmost one (start frequency) in order.   */
/*           Each level point is expressed by an integer from 1792 to 14592. */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_dataTraceIn (ViSession instrSession, ViBoolean memory,
				ViInt16 _VI_FAR traceDataArray[])
{
    ViUInt32 retCnt = 0;
    ViUInt32 i, total = 0;
    ViChar wrtBuf[BUFFER_SIZE*2];
    ViStatus avr3267s_status = VI_SUCCESS;
    avr3267s_instrRange instrPtr;

    if (avr3267s_invalidViBooleanRange (memory))
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "%s<%s", memoryArr[memory]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
   	    return avr3267s_status;
    
    if (avr3267s_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))
        return avr3267s_status;

	total = instrPtr -> traceRange;

	if (total == 1001)
   		Fmt (wrtBuf, "%1001d[zb2o10]<%1001d[b2]", traceDataArray);
   	else
   		Fmt (wrtBuf, "%501d[zb2o10]<%501d[b2]", traceDataArray);
   	
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
   	    return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Trace Data Output                                               */
/* Purpose:  This function returns trace data array. Trace data on the screen*/
/*           includes the data for 1001 or 501 points on frequency axis. For */
/*           outputting these data, it is necessary to transfer data for	 */
/*           1001/501 points from leftmost one (start frequency) in order.   */
/*           Each level point is expressed by an integer from 1792 to 14592. */
/*           (However, when the waveform exceeds the upper limit of the		 */
/*           vertical scale, a value greater than 14592 is transferred.)	 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_dataTraceOut (ViSession instrSession,
                ViBoolean memory, ViInt16 _VI_FAR traceDataArray[])
{
    ViUInt32 retCnt = 0;
    ViUInt32 i, total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViChar rdBuf[BUFFER_SIZE*2];
    ViStatus avr3267s_status = VI_SUCCESS;
    avr3267s_instrRange instrPtr;

    if (avr3267s_invalidViBooleanRange (memory))
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "%s<%s?", memoryArr[memory]);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;
    if (avr3267s_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))
        return avr3267s_status;

    total = instrPtr -> traceRange;

    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE*2, &retCnt)) < 0)
        return avr3267s_status;
    if (total == 1001) {
    	if (Scan (rdBuf, "%1001d[zb2o10]>%1001d[b2]",traceDataArray) < 1)
	    	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
	else {
    	if (Scan (rdBuf, "%501d[zb2o10]>%501d[b2]",traceDataArray) < 1)
	    	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
	
    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Power Meas                                                      */
/* Purpose:  This function selects power measurement mode. Power measurement */
/*           is a function to obtain the power of a signal displayed on the  */
/*           screen and useful for measuring the power of wide band			 */
/*           modulation wave.												 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_dataPowerMeas (ViSession instrSession, ViInt16 powerMeasMode,
                ViPReal64 powerOutput)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (powerMeasMode, 0, 2))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s?", powMeasArr[powerMeasMode]);
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;
    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avr3267s_status;
   	if (Scan (rdBuf, "%s>%f", powerOutput) < 1)
    	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Counter                                                         */
/* Purpose:  Counter function accurately measures the frequency of signal at */
/*           a point where the marker is positioned.						 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_dataCounter (ViSession instrSession, ViPReal64 counterValue)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    Fmt (wrtBuf, "%s<CNRES?");
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;
    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avr3267s_status;
   	if (Scan (rdBuf, "%s>%f", counterValue) < 1)
    	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: OBW Data                                                        */
/* Purpose:  This function returns the OBW (Occupied Bandwidth) parameters of*/
/*           the waveform displayed on the screen. For the result, occupied  */
/*           bandwidth and carrier frequency (Fc) which is equal to the		 */
/*           center frequency in the occupied bandwidth are displayed in the */
/*           result display area on the screen.								 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_dataOBW (ViSession instrSession, ViPReal64 OBW,
				ViPReal64 OBWValue, ViPReal64 carrierFrequency)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;
    
    if ((avr3267s_status = viWrite (instrSession, "OBWPER?", 7, &retCnt)) < 0)        
        return avr3267s_status;                                                    
    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avr3267s_status;                                                    
    if (Scan (rdBuf, "%s>%f", OBW) < 1)          
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		
    if ((avr3267s_status = viWrite (instrSession, "OBW?", 4, &retCnt)) < 0)
        return avr3267s_status;
    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avr3267s_status;
   	if (Scan (rdBuf, "%s>%f,%f", OBWValue, carrierFrequency) < 2)
    	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: ACP Data                                                        */
/* Purpose:  This function returns the ACP (Adjacent Channels Leakage Power) */
/*           data.															 */
/*===========================================================================*/
ViStatus _VI_FUNC CVIFUNC avr3267s_dataACP (ViSession instrSession, ViInt16 *n,
                                   ViReal64 lowerFrequency[], ViReal64 lowerFLevel[],
                                   ViReal64 upperFrequency[], ViReal64 upperFLevel[])
{
	ViUInt32 retCnt = 0;
	ViInt16 i;
	ViChar rdBuf[BUFFER_SIZE];            
	ViStatus avr3267s_status = VI_SUCCESS;
	
	if ((avr3267s_status = viWrite (instrSession, "ACP?", 4, &retCnt)) < 0)                
	    return avr3267s_status;                                                            
	if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)        
	    return avr3267s_status;                                                            
	if (Scan (rdBuf, "%s>%d[b2]", n) < 1)                 
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	
	for (i=0;i<*n;i++) {
		if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
		    return avr3267s_status;                                                    
		if (Scan (rdBuf, "%s>%f,%f,%f,%f", &lowerFrequency[i], &lowerFLevel[i], 
				&upperFrequency[i], &upperFLevel[i]) < 4)                    
		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;                               
	}
	
	return avr3267s_status; 
}

/*===========================================================================*/
/* Function: HARM Data                                                       */
/* Purpose:  This function returns higher harmonics measured values.		 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_dataHARM (ViSession instrSession, ViInt16 *num,
				ViReal64 _VI_FAR frequency[], ViReal64 _VI_FAR level[],
				ViPInt16 levelUnits)
{
    ViUInt32 retCnt = 0;
    ViInt16 i;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;
    avr3267s_instrRange instrPtr;
    
    if (avr3267s_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))
        return avr3267s_status;

    if ((avr3267s_status = viWrite (instrSession, "HARM?", 5, &retCnt)) < 0)
        return avr3267s_status;
    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avr3267s_status;
   	if (Scan (rdBuf, "%s>%d[b2]", num) < 1)                
   	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;     

	for (i=0;i<*num;i++) {
		if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)  
		    return avr3267s_status;                                                      
	    if (Scan (rdBuf, "%s>%f,%f", &frequency[i], &level[i]) < 2)
   		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}

    if ((avr3267s_status = viWrite (instrSession, "AUNITS?", 7, &retCnt)) < 0)
   	    return avr3267s_status;
   	if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avr3267s_status;
    if (Scan (rdBuf, "%s>%d[b2]", levelUnits) < 1)
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Spurious Data                                                   */
/* Purpose:  This function returns the spurious measurement data.			 */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_dataSpurious (ViSession instrSession,
                                        ViInt16 *tableSize,
                                        ViInt16 spuriousNumber[],
                                        void *spuriousFrequency,
                                        void *spuriousLevel, void *judgement)
{
	ViUInt32 retCnt = 0;                     
    ViChar rdBuf[BUFFER_SIZE];               
    ViStatus avr3267s_status = VI_SUCCESS;
    ViInt16 i, j, n;                                               
    ViInt16 *pt_num;
    ViReal64 *pt_freq, *pt_level;
    ViInt16 *pt_jdg;
    
    pt_num = spuriousNumber;
    pt_freq = (ViReal64 *)spuriousFrequency;
	pt_level = (ViReal64 *)spuriousLevel;
	pt_jdg = (ViInt16 *)judgement;
    
    if ((avr3267s_status = viWrite (instrSession, "SPURI?", 6, &retCnt)) < 0)        
        return avr3267s_status;                                                     
    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0) 
        return avr3267s_status;                                                     
    if (Scan (rdBuf, "%s>%d[b2]%d[b2]", tableSize, &n) < 2)                                         
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    for (i=0;i<*tableSize;i++) {
    	*pt_num = n;
    	
    	for (j=0;j<*pt_num;j++) {
    		if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0) 
    		    return avr3267s_status;                                                     
    		if (Scan (rdBuf, "%s>%f,%f,%d[b2]%d[b2]", pt_freq++, pt_level++, pt_jdg++, &n) < 3)          
    		    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    	}
    	
    	pt_num++;
    }
    
	return avr3267s_status;    
}

/*===========================================================================*/ 
/* Function: Noise/Hz Data													 */
/* Purpose:  This function returns Noise/Hz data.							 */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_dataNoiseHz (ViSession instrSession, ViReal64 *powerOutput)
{
	ViUInt32 retCnt = 0;                     
	ViChar rdBuf[BUFFER_SIZE];               
	ViStatus avr3267s_status = VI_SUCCESS;   

	if ((avr3267s_status = viWrite (instrSession, "NIRES?", 6, &retCnt)) < 0)          
	    return avr3267s_status;                                                        
	if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)    
	    return avr3267s_status;                                                        
	if (Scan (rdBuf, "%s>%f", powerOutput) < 1)                            
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;                                   
	
	return avr3267s_status;
}

/*===========================================================================*/
/* Function: % AM Modulation Data											 */
/* Purpose:  This function returns AM modulation data.						 */
/*===========================================================================*/

ViStatus CVIFUNC avr3267s_dataAMMdl (ViSession instrSession, ViReal64 *dataAM)
{
	ViUInt32 retCnt = 0;                                                               
    ViChar rdBuf[BUFFER_SIZE];                                                        
    ViStatus avr3267s_status = VI_SUCCESS;                                            
                                                                                      
    if ((avr3267s_status = viWrite (instrSession, "AMMOD?", 6, &retCnt)) < 0)         
        return avr3267s_status;                                                       
    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)   
        return avr3267s_status;                                                       
    if (Scan (rdBuf, "%s>%f", dataAM) < 1)                                       
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;                                  
                                                                                      
    return avr3267s_status;                                                           
}

/*===========================================================================*/
/* Function: Frequency Data													 */
/* Purpose:  This function returns selected frequency's data.				 */
/*===========================================================================*/
ViStatus CVIFUNC avr3267s_dataFrequency (ViSession instrSession,
                                         ViInt16 select, ViReal64 *frequencyData)
{
	ViUInt32 retCnt = 0;                        
	ViChar wrtBuf[BUFFER_SIZE], rdBuf[BUFFER_SIZE];                  
	ViStatus avr3267s_status = VI_SUCCESS;
	
	if (avr3267s_invalidViInt16Range (select, 0, 3))
		return VI_ERROR_PARAMETER2;
	
	Fmt (wrtBuf, "%s<%s?", freqDataArr[select]);
	if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)   
	    return avr3267s_status;                                                             
	if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)         
	    return avr3267s_status;                                                             
	if (Scan (rdBuf, "%s>%f", frequencyData) < 1)                                            
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;                                        
	
    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Recall Function                                                 */
/* Purpose:  This function stores or recalls data to or from build-in back up*/
/*           memory or a memory card.										 */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_recallFunction (ViSession instrSession,
				ViInt16 recallFunction, ViChar _VI_FAR fileName[])
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if (avr3267s_invalidViInt16Range (recallFunction, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (StringLength (fileName) > 30)
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "%s<%s %s", recallArr[recallFunction], fileName);

    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_writeInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    Fmt (wrtBuf, "%s<%s", writeBuffer);
    
    if ((avr3267s_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Read Instrument Buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_readInstrData (ViSession instrSession, ViInt32 numberBytesToRead,
                ViChar _VI_FAR readBuffer[], ViPInt32 numBytesRead)
{
    ViStatus avr3267s_status = VI_SUCCESS;
    *numBytesRead = 0L;
        
    if ((avr3267s_status = viRead (instrSession, readBuffer, numberBytesToRead, numBytesRead)) < 0)
        return avr3267s_status;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_reset (ViSession instrSession)
{
    ViUInt32 retCnt = 0;
    ViStatus avr3267s_status = VI_SUCCESS;

    /*  Initialize the instrument to a known state.  */
    if ((avr3267s_status = viWrite (instrSession, "IP", 2, &retCnt)) < 0)
        return avr3267s_status;

    if ((avr3267s_status = avr3267s_defaultInstrSetup (instrSession)) < 0)  
        return avr3267s_status;
        
    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. If the self test function is not supported by the   */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_SELF_TEST.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_selfTest (ViSession instrSession, ViPInt16 testResult,
                    ViChar _VI_FAR testMessage[])
{
    ViStatus avr3267s_status = VI_SUCCESS;

    Fmt (testMessage, "\"Instrument Does Not support a Self Test\"");
    
    *testResult = 0;
    
    avr3267s_status = VI_WARN_NSUP_SELF_TEST;

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_errorQuery (ViSession instrSession, ViPInt32 errorCode,
                ViChar _VI_FAR errorMessage[])
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];    
    ViStatus avr3267s_status = VI_SUCCESS;

    if ((avr3267s_status = viWrite (instrSession, "ERRNO?", 6, &retCnt)) < 0)
        return avr3267s_status;

    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avr3267s_status;
    
    if (Scan (rdBuf, "%s>%d", errorCode) < 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	Fmt (errorMessage, "\"\"");

    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_errorMessage (ViSession instrSession, ViStatus statusCode,
                ViChar _VI_FAR errMessage[])
{
    ViStatus avr3267s_status = VI_SUCCESS;
    ViInt16 i;
    static avr3267s_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,     "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,        "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,  "WARNING: Error Query not supported"},     
        {VI_WARN_NSUP_REV_QUERY,    "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,    "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,   "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_INSTR_PARAMETER9 ,  "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10,  "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11,  "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12,  "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13,  "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14,  "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15,  "ERROR: Parameter 15 out of range"},
        {VI_ERROR_INSTR_PARAMETER16,  "ERROR: Parameter 16 out of range"},
        {avr3267s_ERROR_INVALID_CONFIGURATION, "ERROR: Instrument configuration error"},
        
        {VI_NULL, VI_NULL}
    };

    avr3267s_status = viStatusDesc (instrSession, statusCode, errMessage);
    if (avr3267s_status == VI_WARN_UNKNOWN_STATUS) {
        for (i = 0; statusDescArray[i].stringName; i++) {
            if (statusDescArray[i].stringVal == statusCode) {
                Fmt (errMessage, "%s<%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        Fmt (errMessage, "%s<Unknown Error 0x%x[uw8p0]", statusCode);
        return (VI_WARN_UNKNOWN_STATUS);
    }
    
    avr3267s_status = VI_SUCCESS;
    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*           If the revision query function is not supported by the          */ 
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_REV_QUERY.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_revisionQuery (ViSession instrSession,
                ViChar _VI_FAR driverRev[], ViChar _VI_FAR instrRev[])
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avr3267s_status = VI_SUCCESS;

    if ((avr3267s_status = viWrite (instrSession, "*IDN?", 5, &retCnt)) < 0)
        return avr3267s_status;

    if ((avr3267s_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return avr3267s_status;

    if (Scan (rdBuf, "%s>%s[dt44],%s[dt44],%s[dt44],%s", instrRev) < 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    Fmt (driverRev, "%s<%s", avr3267s_REVISION);
    
    return avr3267s_status;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC avr3267s_close (ViSession instrSession)
{
    avr3267s_instrRange instrPtr;
    ViSession rmSession;
    ViStatus avr3267s_status = VI_SUCCESS;

    if ((avr3267s_status = viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession)) < 0)
        return avr3267s_status;
    if ((avr3267s_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return avr3267s_status;
            
    if (instrPtr != NULL) 
        free (instrPtr);
    
    avr3267s_status = viClose (instrSession);
    viClose (rmSession);

    return avr3267s_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean avr3267s_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avr3267s_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avr3267s_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Unsigned Char Value Out Of Range - ViUInt8                      */
/* Purpose:  This function checks an unsigned char value to see if it        */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avr3267s_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avr3267s_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avr3267s_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avr3267s_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avr3267s_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*================================================================================*/
/* Function: Wait On VISA Event                                                   */
/* Purpose:  This function waits for the occurrence of an enabled VISA Event. The */
/*           event type must be enabled before entering this function. Any        */
/*           programmatic commands, actions, or conditions, necessary to generate */
/*           the specified event type, must be satisified before entering this    */
/*           function. The function will wait for the specified timeout and then  */
/*           return. If the specified event is received then the function will    */
/*           return VI_SUCCESS (0), otherwise the status code for the generated   */
/*           error will be returned. In either case the function will disable the */
/*           event type and deallocate the event handle passed from viWaitOnEvent */
/*           before returning. WARNING: If VI_TMO_INFINITE is passed in as the    */
/*           timeout this function WILL NOT return until the specified event is   */
/*           received, if the event is not received the function will not return  */
/*           and it will be necessary to terminate CVI in order to regain control.*/
/*================================================================================*/
ViStatus avr3267s_waitOnVisaEvent (ViSession instrSession, ViEventType eventTypeIn, 
            ViUInt32 timeoutIn, ViPUInt16 STB)
{            
    ViStatus avr3267s_status = VI_SUCCESS, tempStatus = VI_SUCCESS;
    ViEventType eventTypeOut = 0;
    ViEvent eventHandle = 0;
    
    /* For debug purposes we want to be able to see the status returned by        */
    /* viDisableEvent and viClose if one of the internal functions fails but do   */
    /* not want to return that value as that is not where the initial error       */
    /* occurs in the function, so we assign it to tempStatus.                     */
    
    if ((avr3267s_status = viWaitOnEvent (instrSession, eventTypeIn, timeoutIn, 
                                        &eventTypeOut, &eventHandle)) < 0) {
        tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
        return avr3267s_status;
    } 
    
    if (eventTypeIn == VI_EVENT_SERVICE_REQ)
        if ((avr3267s_status = viReadSTB (instrSession, STB)) < 0) {
            tempStatus = viClose (eventHandle);
            tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
            return avr3267s_status;
        }   
        
    if ((avr3267s_status = viClose (eventHandle)) < 0) {             
        tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
        return avr3267s_status;                 
    }
    
    if ((avr3267s_status = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE)) < 0)
        return avr3267s_status;                                                   

    return avr3267s_status;                                    
}    

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the avr3267s_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from avr3267s_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus avr3267s_initCleanUp (ViSession openRMSession,
                ViPSession openInstrSession, ViStatus currentStatus)
{
    avr3267s_instrRange instrPtr;
    
    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL) 
            free (instrPtr);

    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the avr3267s_reset operation and by the     */
/*           avr3267s_init function if the reset option has not been           */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus avr3267s_defaultInstrSetup (ViSession instrSession)
{
    ViStatus avr3267s_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    avr3267s_instrRange instrPtr;
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    if (avr3267s_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))
        return avr3267s_status;
    
    if (instrPtr == NULL) 
        instrPtr = malloc (sizeof (struct avr3267s_statusDataRanges));

    instrPtr -> traceRange = 1001;
    instrPtr -> harmonics = 3;
    Fmt (instrPtr -> instrDriverRevision, "%s<%s", avr3267s_REVISION);
    
    if ((avr3267s_status = viSetAttribute (instrSession, VI_ATTR_USER_DATA, 
                            (ViUInt32)instrPtr)) < 0)
        return avr3267s_status;                                       

    return avr3267s_status;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
