#include <cvidef.h>

/*= R3267s Series - Modulation Spectrum Analyzes Include File ===============*/

/*===========================================================================*/
/*  Please do not use global variables or arrays in the include file of      */
/*  instrument drivers that will be submitted for inclusion into the         */
/*  LabWindows Instrument Driver Library.                                    */
/*===========================================================================*/
     
#ifndef __avr3267s_HEADER
#define __avr3267s_HEADER

#include <vpptype.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************/
/*= Instrument Driver Specific Error/Warning Codes ==========================*/
/*****************************************************************************/
#define VI_ERROR_INSTR_FILE_OPEN                (_VI_ERROR+0x3FFC0800L)
#define VI_ERROR_INSTR_FILE_WRITE               (_VI_ERROR+0x3FFC0801L)
#define VI_ERROR_INSTR_INTERPRETING_RESPONSE    (_VI_ERROR+0x3FFC0803L)
#define VI_ERROR_INSTR_PARAMETER9               (_VI_ERROR+0x3FFC0809L)
#define VI_ERROR_INSTR_PARAMETER10              (_VI_ERROR+0x3FFC080AL)
#define VI_ERROR_INSTR_PARAMETER11              (_VI_ERROR+0x3FFC080BL)
#define VI_ERROR_INSTR_PARAMETER12              (_VI_ERROR+0x3FFC080CL)
#define VI_ERROR_INSTR_PARAMETER13              (_VI_ERROR+0x3FFC080DL)
#define VI_ERROR_INSTR_PARAMETER14              (_VI_ERROR+0x3FFC080EL)
#define VI_ERROR_INSTR_PARAMETER15              (_VI_ERROR+0x3FFC080FL)
#define VI_ERROR_INSTR_PARAMETER16              (_VI_ERROR+0x3FFC0810L)

/*****************************************************************************/
/*= Define Instrument Specific Error/Warning Codes Here =====================*/
/*****************************************************************************/
#define VI_WARNING_INSTR_OFFSET                           (0x3FFC0900L)
#define VI_ERROR_INSTR_OFFSET                   (_VI_ERROR+0x3FFC0900L)

#define avr3267s_ERROR_INVALID_CONFIGURATION      (VI_ERROR_INSTR_OFFSET + 0xF0L)

/*****************************************************************************/
/*= GLOBAL USER-CALLABLE FUNCTION DECLARATIONS (Exportable Functions) =======*/
/*****************************************************************************/
ViStatus _VI_FUNC avr3267s_init (ViRsrc resourceName, ViBoolean IDQuery,
                ViBoolean resetDevice, ViPSession instrSession);
ViStatus _VI_FUNC avr3267s_appFrqMeas (ViSession instrSession,
                ViReal64 centerFrequencyHz, ViReal64 frequencySpanHz,
                ViPReal64 markerFrequencyHz, ViPReal64 markerLevel,
                ViInt16 _VI_FAR traceDataArray[]);
ViStatus _VI_FUNC avr3267s_setOperationMode (ViSession instrumentHandle,
                                             ViBoolean operationMode);
ViStatus _VI_FUNC avr3267s_setCommunicationSystem (ViSession instrumentHandle,
                                                   ViInt16 communicationSystem);
ViStatus _VI_FUNC avr3267s_confCenterFrq (ViSession instrSession,
                ViReal64 centerFrequencyHz, ViBoolean CFStep,
                ViReal64 CFStepSizeHz, ViBoolean freqOffset, ViReal64 freqOffsetSizeHz);
ViStatus _VI_FUNC avr3267s_confFrqSpan (ViSession instrSession, ViInt16 spanMode,
                ViReal64 frequencySpanHz);
ViStatus _VI_FUNC avr3267s_confStartStopFrq (ViSession instrSession,
                ViReal64 startFrequencyHz, ViReal64 stopFrequencyHz);
ViStatus _VI_FUNC avr3267s_confFrqPreSel (ViSession instrSession,
                ViInt16 preselector, ViBoolean peaking, ViInt32 manualPeakingValue);
ViStatus CVIFUNC avr3267s_confFreqSetup (ViSession instrSession,
                                         ViInt16 contents, ViReal64 center_start,
                                         ViReal64 span_stop);
ViStatus _VI_FUNC avr3267s_confRefLevel (ViSession instrSession,
                ViReal64 referenceLeveldBm, ViInt16 xdBdiv, ViInt16 linear,
                ViInt16 units, ViBoolean levelOffset, ViReal64 levelOffsetValuedB);
ViStatus _VI_FUNC avr3267s_confSweepMode (ViSession instrSession,
                ViBoolean sweepMode, ViBoolean window, ViReal64 gatePositions,
                ViReal64 gateWidths, ViBoolean gatedSweep, ViInt16 gateSource);
ViStatus _VI_FUNC avr3267s_confSweepTime (ViSession instrSession,
                ViBoolean sweepTimeMode, ViReal64 sweepTimeValues);
ViStatus _VI_FUNC avr3267s_confTriggerMode (ViSession instrSession, ViInt16 triggerMode, 
				ViInt16 videoTrgLevel, ViBoolean triggerSlope, 
				ViBoolean triggerDelay, ViReal64 delayTime);
ViStatus _VI_FUNC avr3267s_confResBW (ViSession instrSession, ViBoolean RBWMode,
                ViReal64 RBWFrqValueHz, ViBoolean RBWSpan, ViReal64 RBWSpanValue);
ViStatus _VI_FUNC avr3267s_confVideoBW (ViSession instrSession,
                ViBoolean VBWMode, ViReal64 VBWFrqValueHz, ViBoolean VBWSpan,
                ViReal64 VBWSpanValue);
ViStatus _VI_FUNC avr3267s_confATT (ViSession instrSession, ViBoolean ATTLevel,
                ViReal64 ATTLevelValuedB, ViBoolean minimumATT, ViReal64 minATTValuedB);
ViStatus _VI_FUNC avr3267s_confTraceDetector (ViSession instrSession,
                ViBoolean trace, ViInt16 detectorMode);
ViStatus _VI_FUNC avr3267s_confTraceRange (ViSession instrSession,
                ViBoolean measurementPoints);
ViStatus _VI_FUNC avr3267s_confMkrMode (ViSession instrSession,
                ViInt16 markerMode, ViReal64 markerFrequencyHz,
                ViInt16 markerStep, ViReal64 stepSize);
ViStatus _VI_FUNC avr3267s_confOthMkrFunc (ViSession instrSession,
                ViInt16 markerFunctions);
ViStatus _VI_FUNC avr3267s_confSigTrack (ViSession instrSession,
                ViBoolean signalTrack);
ViStatus CVIFUNC avr3267s_confMkrRefObj (ViSession instrSession, 
				ViInt16 referenceObject);
ViStatus _VI_FUNC avr3267s_confPrinter (ViSession instrSession, ViInt16 copyMode,
                ViInt16 printerCommand);
ViStatus _VI_FUNC avr3267s_confBMP (ViSession instrSession, ViInt16 imageMode,
                ViBoolean RLECompression, ViInt16 fileNo);
ViStatus CVIFUNC avr3267s_confAutoManualSetup (ViSession instrSession,
                                               ViInt16 contents, ViBoolean mode,
                                               ViReal64 manualValue);
ViStatus CVIFUNC avr3267s_confMixerSetup (ViSession instrSession,
                                          ViBoolean mixer, ViInt16 bandN,
                                          ViBoolean bandLock, ViBoolean avgLoss,
                                          ViReal64 avgLossValue,
                                          ViBoolean lossvsFreq);
ViStatus CVIFUNC avr3267s_confMixerBias (ViSession instrSession, 
										ViReal64 mixerBiasValue);
ViStatus CVIFUNC avr3267s_confMixerTable (ViSession instrSession,
                                          ViInt16 tableSize, ViReal64 frequency[],
                                          ViReal64 level[], ViReal64 bias[]);
ViStatus _VI_FUNC avr3267s_actSweep (ViSession instrSession, ViInt16 sweepAction);
ViStatus _VI_FUNC avr3267s_actAutoCpl (ViSession instrSession);
ViStatus _VI_FUNC avr3267s_actTraceA (ViSession instrSession,
                ViInt16 modesofTraceA, ViInt16 averagingValue);
ViStatus _VI_FUNC avr3267s_actTraceB (ViSession instrSession,
                ViInt16 modesofTraceB);
ViStatus CVIFUNC avr3267s_actNormalizeA (ViSession instrSession,
                                         ViBoolean normA, ViBoolean storeCorrData);
ViStatus CVIFUNC avr3267s_actNormalizeB (ViSession instrSession,
                                         ViBoolean NormB, ViBoolean storeCorrData);
ViStatus CVIFUNC avr3267s_actTraceExchange (ViSession instrSession);
ViStatus _VI_FUNC avr3267s_actDisplayLine (ViSession instrSession,
                ViBoolean displayLine, ViReal64 lineLeveldB);
ViStatus _VI_FUNC CVIFUNC avr3267s_actLimitLine (ViSession instrSession,
										ViBoolean judge,
                                        ViBoolean limitLine1, ViBoolean line1PassRange,
                                        ViBoolean limitLine2, ViBoolean line2PassRange,
                                        ViBoolean XDataMode, ViInt16 XReference,
                                        ViReal64 XUserDefine, ViReal64 XOffset,
                                        ViBoolean YDataMode, ViInt16 YReference,
                                        ViReal64 YUserDefine, ViReal64 YOffset);
ViStatus _VI_FUNC avr3267s_actLimLineTbl (ViSession instrSession,
                ViInt16 tableOperation, ViBoolean tableMode, ViReal64 frequencyValueHz, 
                ViReal64 levelValuedB);
ViStatus _VI_FUNC avr3267s_actLimLineJudg (ViSession instrSession,
                ViInt16 judgmentOperation, ViPInt16 judgmentResult, ViPInt16 noofFailPoints,
                ViReal64 _VI_FAR frequencyFailPoints[],
                ViReal64 _VI_FAR levelFailPoints[]);
ViStatus _VI_FUNC avr3267s_actMkrData (ViSession instrSession,
                ViPReal64 frequencyHz, ViPReal64 level, ViPInt16 levelUnits);
ViStatus CVIFUNC avr3267s_actDltMkrData (ViSession instrSession,
                                         ViInt16 markerKind, ViReal64 *x,
                                         ViReal64 *level);
ViStatus _VI_FUNC avr3267s_actPeakSearch (ViSession instrSession,
                ViInt16 peakSearchAction, ViBoolean contPeak);
ViStatus CVIFUNC avr3267s_actPkSchCond (ViSession instrSession,
                                        ViInt16 xAxisPeakRange,
                                        ViReal64 limitPosition,
                                        ViReal64 limitWidth, ViBoolean coupletoFT,
                                        ViInt16 yAxisPeakRange,
                                        ViReal64 peakDeltaY);
ViStatus _VI_FUNC avr3267s_actMultiMarker (ViSession instrSession,
                ViInt16 multiMarkerMode, ViBoolean multiMarkerStatus,
                ViReal64 frequencyValueHz);
ViStatus _VI_FUNC avr3267s_actMultiMkrData (ViSession instrSession,
                ViReal64 _VI_FAR frequencyHz[], ViReal64 _VI_FAR level[],
                ViPInt16 levelUnits);
ViStatus _VI_FUNC avr3267s_actPeakList (ViSession instrSession,
                ViInt16 peakListMode, ViReal64 _VI_FAR frequencyHz[],
                ViReal64 _VI_FAR level[], ViPInt16 levelUnits);
ViStatus _VI_FUNC avr3267s_actMarkerTo (ViSession instrSession, ViInt16 marker);
ViStatus _VI_FUNC avr3267s_actdBDown (ViSession instrSession, ViInt16 dBDown,
                ViReal64 dBDownWidthdB);
ViStatus CVIFUNC avr3267s_act3rdOrder (ViSession instrSession);
ViStatus CVIFUNC avr3267s_actAMMdl (ViSession instrSession, ViInt16 modAMMeas);
ViStatus _VI_FUNC avr3267s_actMeasWindow (ViSession instrSession,
                ViBoolean windowState, ViReal64 centerPositionHz, ViReal64 windowWidthHz,
                ViBoolean windowSweep);
ViStatus _VI_FUNC avr3267s_actMultiScreen (ViSession instrSession,
                ViInt16 multiScreenMode, ViReal64 centerPositionHz, ViReal64 windowWidthHz,
                ViInt16 peakZoom, ViBoolean screenSide);
ViStatus _VI_FUNC avr3267s_actInputCorr (ViSession instrSession,
                ViInt16 correctionMode, ViReal64 frequencyCorrDataHz,
                ViReal64 levelCorrDatadB);
ViStatus _VI_FUNC avr3267s_actHardCopy (ViSession instrSession,
                ViInt16 deviceSelection);
ViStatus _VI_FUNC avr3267s_actCalibration (ViSession instrSession,
                ViInt16 calibrationFunction, ViReal64 calibrationLeveldB,
                ViInt16 correctValue, ViBoolean freqCorr, ViBoolean calCorr);
ViStatus _VI_FUNC avr3267s_actLabel (ViSession instrSession,
                ViBoolean labelStaus, ViChar _VI_FAR labelString[]);
ViStatus _VI_FUNC avr3267s_actPowerMeas (ViSession instrSession,
                ViInt16 powerMeasMode, ViInt16 averageTime);
ViStatus _VI_FUNC avr3267s_actCounter (ViSession instrSession,
                ViBoolean counterMode, ViInt16 resolution);
ViStatus _VI_FUNC avr3267s_actSound (ViSession instrSession, ViInt16 soundMode,
                ViInt16 volume, ViReal64 pauseTimes, ViBoolean squelchMode,
                ViReal64 squelchValuedB);
ViStatus _VI_FUNC avr3267s_actNoiseHz (ViSession instrSession,
                ViInt16 noiseHzMode, ViReal64 noiseFrqHz);
ViStatus _VI_FUNC avr3267s_actOBW (ViSession instrSession, ViInt16 setup, 
				ViReal64 OBW, ViInt16 averageTime);
ViStatus _VI_FUNC CVIFUNC avr3267s_actACP (ViSession instrumentHandle, ViInt16 ACPSetUp,
                                  ViBoolean ACPScreen, ViBoolean ACPGraph,
                                  ViReal64 carrierbandWidth, ViInt16 ACPNumber,
                                  ViReal64 ACPCSHz[], ViReal64 ACPBSHz[],
                                  ViInt16 averageTime);
ViStatus CVIFUNC avr3267s_actACPFilter (ViSession instrSession, ViInt16 filterControl,
                                        ViReal64 symbolRate, ViReal64 rolloffFactor);
ViStatus _VI_FUNC avr3267s_actHARM (ViSession instrSession, ViBoolean slctFund, 
									ViReal64 fundFreq, ViInt16 harmonicNo);
ViStatus CVIFUNC avr3267s_actSprMeas (ViSession instrSession, ViBoolean sprMeasMode);
ViStatus CVIFUNC avr3267s_actSprTbl (ViSession instrSession, ViInt16 tableNo,
                                     ViBoolean loadTable, ViBoolean saveTable,
                                     ViInt16 tableSize,
                                     ViReal64 startFrequency[],
                                     ViReal64 stopFrequency[], ViReal64 RBW[],
                                     ViReal64 limitLevel[]);
ViStatus CVIFUNC avr3267s_actArtificialAnalog (ViSession instrSession, 
											   ViBoolean artificialAnalog,
                                               ViBoolean traceDisplay,
                                               ViInt16 samplingTimes);
ViStatus CVIFUNC avr3267s_actXYCursor (ViSession instrSession,
                                       ViBoolean XYCursor, ViInt16 anchor,
                                       ViReal64 XCursorPosition,
                                       ViReal64 YCursorPosition);
ViStatus _VI_FUNC avr3267s_actStdOSR (ViSession instrSession,
                ViBoolean registerMode, ViInt32 registerValue,
                ViPInt32 registerStatus);
ViStatus _VI_FUNC avr3267s_actStdER (ViSession instrSession,
                ViInt16 registerMode, ViInt16 registerValue,
                ViPInt16 registerStatus);
ViStatus _VI_FUNC avr3267s_actSTB (ViSession instrSession,
                ViBoolean registerMode, ViInt16 registerValue,
                ViPInt16 registerStatus);
ViStatus CVIFUNC avr3267s_actServiceRequest (ViSession instrSession,
                                             ViBoolean serviceRequest, 
                                             ViInt16 dataSRE);
ViStatus CVIFUNC avr3267s_actSetMaskBit_ClrSTB (ViSession instrSession,
                                                ViInt32 operMaskBit, 
                                                ViInt32 eventMaskBit);
ViStatus CVIFUNC avr3267s_actWaitMeas (ViSession instrSession,
                                       ViInt32 maxWaitTime, ViBoolean *errorFlag);
ViStatus _VI_FUNC avr3267s_dataTraceIn (ViSession instrSession, ViBoolean memory,
                ViInt16 _VI_FAR traceDataArray[]);
ViStatus _VI_FUNC avr3267s_dataTraceOut (ViSession instrSession,
                ViBoolean memory, ViInt16 _VI_FAR traceDataArray[]);
ViStatus _VI_FUNC avr3267s_dataPowerMeas (ViSession instrSession,
                ViInt16 powerMeasMode, ViPReal64 powerOutput);
ViStatus _VI_FUNC avr3267s_dataCounter (ViSession instrSession,
                ViPReal64 counterValue);
ViStatus _VI_FUNC avr3267s_dataOBW (ViSession instrSession, ViPReal64 OBW,
                ViPReal64 OBWValueHz, ViPReal64 carrierFrequencyHz);
ViStatus _VI_FUNC CVIFUNC avr3267s_dataACP (ViSession instrSession, ViInt16 *ACPSize,
                                   ViReal64 lowerFrequency[], ViReal64 lowerFLevedl[],
                                   ViReal64 upperFrequency[], ViReal64 upperFLevel[]);
ViStatus _VI_FUNC avr3267s_dataHARM (ViSession instrSession, ViInt16 *HARMSize,
                ViReal64 _VI_FAR frequencyHz[], ViReal64 _VI_FAR level[],
                ViPInt16 levelUnits);
ViStatus CVIFUNC avr3267s_dataSpurious (ViSession instrSession,
                                        ViInt16 *tableSize,
                                        ViInt16 numberofSpurious[],
                                        void *spuriousFrequency,
                                        void *spuriousLevel, void *judgement);
ViStatus CVIFUNC avr3267s_dataNoiseHz (ViSession instrSession, ViReal64 *powerOutput);
ViStatus CVIFUNC avr3267s_dataAMMdl (ViSession instrSession, ViReal64 *dataAM);
ViStatus CVIFUNC avr3267s_dataFrequency (ViSession instrSession,
                                         ViInt16 select, ViReal64 *frequencyData);
ViStatus _VI_FUNC avr3267s_recallFunction (ViSession instrSession,
                ViInt16 recallFunction, ViChar _VI_FAR fileName[]);
ViStatus _VI_FUNC avr3267s_writeInstrData (ViSession instrSession,
                ViString writeBuffer);
ViStatus _VI_FUNC avr3267s_readInstrData (ViSession instrSession,
                ViInt32 numberBytesToRead, ViChar _VI_FAR readBuffer[],
                ViPInt32 numBytesRead);
ViStatus _VI_FUNC avr3267s_reset (ViSession instrSession);
ViStatus _VI_FUNC avr3267s_selfTest (ViSession instrSession,
                ViPInt16 selfTestResult, ViChar _VI_FAR selfTestMessage[]);
ViStatus _VI_FUNC avr3267s_errorQuery (ViSession instrSession,
                ViPInt32 errorCode, ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC avr3267s_errorMessage (ViSession instrSession,
                ViStatus statusCode, ViChar _VI_FAR message[]);
ViStatus _VI_FUNC avr3267s_revisionQuery (ViSession instrSession,
                ViChar _VI_FAR instrumentDriverRevision[], 
                ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC avr3267s_close (ViSession instrSession);

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif

/*****************************************************************************/
/*=== END INCLUDE FILE ======================================================*/
/*****************************************************************************/

#endif
